from django.core.exceptions import ValidationError

from mayan.apps.documents.tests.mixins.document_mixins import DocumentTestMixin
from mayan.apps.documents.permissions import permission_document_view
from mayan.apps.testing.tests.base import BaseTestCase

from ..models import Cabinet

from .literals import TEST_CABINET_LABEL
from .mixins import CabinetTestMixin


class CabinetTestCase(CabinetTestMixin, BaseTestCase):
    def test_cabinet_creation(self):
        self._create_test_cabinet()

        self.assertEqual(Cabinet.objects.all().count(), 1)
        self.assertQuerysetEqual(
            Cabinet.objects.all(), (repr(self.test_cabinet),)
        )

    def test_cabinet_duplicate_creation(self):
        self._create_test_cabinet()

        with self.assertRaises(expected_exception=ValidationError):
            cabinet_2 = Cabinet(label=TEST_CABINET_LABEL)
            cabinet_2.validate_unique()
            cabinet_2.save()

        self.assertEqual(Cabinet.objects.all().count(), 1)
        self.assertQuerysetEqual(
            Cabinet.objects.all(), (repr(self.test_cabinet),)
        )

    def test_inner_cabinet_creation(self):
        self._create_test_cabinet()

        inner_cabinet = Cabinet.objects.create(
            parent=self.test_cabinet, label=TEST_CABINET_LABEL
        )

        self.assertEqual(Cabinet.objects.all().count(), 2)
        self.assertQuerysetEqual(
            Cabinet.objects.all(),
            map(repr, (self.test_cabinet, inner_cabinet))
        )

    def test_method_get_absolute_url(self):
        self._create_test_cabinet()

        self.assertTrue(self.test_cabinet.get_absolute_url())


class CabinetDocumentTestCase(
    CabinetTestMixin, DocumentTestMixin, BaseTestCase
):
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()
        self._create_test_document_stub()
        self._create_test_cabinet()

    def test_addition_of_documents(self):
        self.test_cabinet.documents.add(self.test_document)

        self.assertEqual(self.test_cabinet.documents.count(), 1)
        self.assertQuerysetEqual(
            self.test_cabinet.documents.all(), (repr(self.test_document),)
        )

    def test_addition_and_deletion_of_documents(self):
        self.test_cabinet.documents.add(self.test_document)

        self.assertEqual(self.test_cabinet.documents.count(), 1)
        self.assertQuerysetEqual(
            self.test_cabinet.documents.all(), (repr(self.test_document),)
        )

        self.test_cabinet.documents.remove(self.test_document)

        self.assertEqual(self.test_cabinet.documents.count(), 0)
        self.assertQuerysetEqual(self.test_cabinet.documents.all(), ())

    def test_cabinet_get_document_count_method(self):
        self.test_cabinet.documents.add(self.test_document)

        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        self.assertEqual(
            self.test_cabinet.get_document_count(user=self._test_case_user),
            len(self.test_documents)
        )

    def test_trashed_document_cabinet_document_count_method(self):
        self.test_cabinet.documents.add(self.test_document)
        self.test_document.delete()

        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        self.assertEqual(
            self.test_cabinet.get_document_count(user=self._test_case_user),
            len(self.test_documents) - 1
        )



import pyotp

from django import forms
from django.contrib.auth import authenticate, get_user_model
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _

import mayan
from mayan.apps.authentication.forms import AuthenticationFormBase
from mayan.apps.authentication.literals import SESSION_MULTI_FACTOR_USER_ID_KEY
from mayan.apps.converter.fields import QRCodeImageField
from mayan.apps.views.forms import DetailForm

from .models import UserOTPData


class AuthenticationFormTOTP(AuthenticationFormBase):
    error_messages = {
        'invalid_token': _(
            'Token is either invalid or expired.'
        )
    }

    token = forms.CharField(
        label=_('TOTP token'), widget=forms.TextInput(
            attrs={
                'autocomplete': 'one-time-code', 'autofocus': True,
                'inputmode': 'numeric'
            }
        )
    )

    @classmethod
    def condition(cls, authentication_backend, wizard):
        user_id = wizard.request.session.get(
            SESSION_MULTI_FACTOR_USER_ID_KEY, None
        )

        if user_id:
            user = get_user_model().objects.get(pk=user_id)
            kwargs = {
                'user__{}'.format(
                    authentication_backend.login_form_class.PASSWORD_FIELD
                ): user.username
            }

            try:
                otp_data = UserOTPData.objects.get(**kwargs)
            except UserOTPData.DoesNotExist:
                return False
            else:
                return otp_data.is_enabled()
        else:
            return False

    def clean(self):
        user_id = self.request.session.get(SESSION_MULTI_FACTOR_USER_ID_KEY, None)

        if user_id:
            user = get_user_model().objects.get(pk=user_id)
            self.user_cache = authenticate(
                factor_name='otp_token',
                otp_token=self.cleaned_data.get('token'),
                request=self.request, user=user
            )
            if self.user_cache is None:
                raise forms.ValidationError(
                    self.error_messages['invalid_token'],
                    code='invalid_token',
                )

        return self.cleaned_data


class FormUserOTPDataDetail(DetailForm):
    def __init__(self, *args, **kwargs):
        instance = kwargs['instance']
        extra_fields = ()

        otp_enabled = instance.otp_data.is_enabled()

        extra_fields = (
            {
                'label': _('OTP enabled?'),
                'func': lambda instance: _('Yes') if otp_enabled else _('No')
            },
        )

        kwargs['extra_fields'] = extra_fields
        super().__init__(*args, **kwargs)

    class Meta:
        fields = ()
        model = UserOTPData


class FormUserOTPDataEdit(forms.Form):
    qr_code = QRCodeImageField(disabled=True, label='', required=False)
    secret = forms.CharField(
        disabled=True,
        help_text=_(
            'Scan the QR code or enter the secret in your authentication '
            'device. Do not share this secret, treat it like a password.'
        ), label=_('Secret'), required=False, widget=forms.TextInput(
            attrs={'readonly': 'readonly'}
        )
    )
    signed_secret = forms.CharField(
        label=_('Secret'), required=False, widget=forms.HiddenInput(
            attrs={'readonly': 'readonly'}
        )
    )
    token = forms.CharField(
        help_text=_(
            'Enter the corresponding token to validate that the secret '
            'was saved correct.'
        ),
        label=_('Token'), widget=forms.TextInput(
            attrs={
                'autocomplete': 'one-time-code', 'autofocus': True,
                'inputmode': 'numeric'
            }
        )
    )

    def __init__(self, *args, **kwargs):
        user = kwargs.pop('user')

        super().__init__(*args, **kwargs)

        secret = self.initial['secret']
        if secret:
            topt = pyotp.totp.TOTP(s=secret)
            url = topt.provisioning_uri(
                name=user.email, issuer_name=mayan.__title__
            )

            self.fields['qr_code'].initial = url

        self.fields['qr_code'].widget.attrs.update(
            {'style': 'margin:auto;'}
        )

    def clean_token(self):
        secret = self.cleaned_data['secret']
        token = self.cleaned_data['token']

        totp = pyotp.TOTP(secret)

        if token.strip() != totp.now():
            raise ValidationError(
                _('Token is incorrect for the specified secret.'),
                code='token_invalid'
            )

        return token



from django.db import migrations


def code_revert_module_paths(apps, schema_editor):
    MetadataType = apps.get_model(
        app_label='metadata', model_name='MetadataType'
    )

    for metadata_type in MetadataType.objects.all():
        metadata_type.parser = metadata_type.parser.replace(
            'mayan.apps.metadata.metadata_parsers',
            'mayan.apps.metadata.parsers'
        )
        metadata_type.validation = metadata_type.validation.replace(
            'mayan.apps.metadata.metadata_validators',
            'mayan.apps.metadata.validators'
        )
        metadata_type.save()


def code_update_module_paths(apps, schema_editor):
    MetadataType = apps.get_model(
        app_label='metadata', model_name='MetadataType'
    )

    for metadata_type in MetadataType.objects.all():
        metadata_type.parser = metadata_type.parser.replace(
            'mayan.apps.metadata.parsers',
            'mayan.apps.metadata.metadata_parsers'
        )
        metadata_type.validation = metadata_type.validation.replace(
            'mayan.apps.metadata.validators',
            'mayan.apps.metadata.metadata_validators'
        )
        metadata_type.save()


class Migration(migrations.Migration):
    dependencies = [
        ('metadata', '0015_metadatatype_validation_arguments')
    ]

    operations = [
        migrations.RunPython(
            code=code_update_module_paths,
            reverse_code=code_revert_module_paths
        )
    ]



from mayan.apps.rest_api import serializers

from .models import DocumentVersionPageOCRContent, DocumentTypeOCRSettings


class DocumentVersionPageOCRContentSerializer(serializers.ModelSerializer):
    class Meta:
        fields = ('content',)
        model = DocumentVersionPageOCRContent
        read_only_fields = ()


class DocumentTypeOCRSettingsSerializer(serializers.ModelSerializer):
    class Meta:
        fields = ('auto_ocr',)
        model = DocumentTypeOCRSettings
        read_only_fields = ()



from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.apps import MayanAppConfig

from .classes import ClientBackend


class PlatformApp(MayanAppConfig):
    app_namespace = 'platform'
    app_url = 'platform'
    has_rest_api = False
    has_tests = True
    name = 'mayan.apps.platform'
    verbose_name = _('Platform')

    def ready(self):
        super().ready()

        ClientBackend.load_modules()



from rest_framework import status

from mayan.apps.documents.tests.mixins.document_mixins import DocumentTestMixin
from mayan.apps.rest_api.tests.base import BaseAPITestCase

from ..events import event_workflow_template_edited
from ..permissions import (
    permission_workflow_template_edit, permission_workflow_template_view
)

from .mixins.workflow_template_mixins import WorkflowTemplateTestMixin
from .mixins.workflow_template_transition_mixins import (
    WorkflowTransitionFieldAPIViewTestMixin, WorkflowTransitionFieldTestMixin
)


class WorkflowTemplateTransitionFieldAPIViewTestCase(
    WorkflowTransitionFieldAPIViewTestMixin, DocumentTestMixin,
    WorkflowTemplateTestMixin, WorkflowTransitionFieldTestMixin, BaseAPITestCase
):
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()
        self._create_test_workflow_template()
        self._create_test_workflow_template_state()
        self._create_test_workflow_template_state()
        self._create_test_workflow_template_transition()

    def test_workflow_template_transition_field_create_api_view_no_permission(self):
        transition_field_count = self._test_workflow_template_transition.fields.count()

        self._clear_events()

        response = self._request_test_workflow_template_transition_field_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self._test_workflow_template_transition.refresh_from_db()
        self.assertEqual(
            self._test_workflow_template_transition.fields.count(),
            transition_field_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_transition_field_create_api_view_with_access(self):
        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        transition_field_count = self._test_workflow_template_transition.fields.count()

        self._clear_events()

        response = self._request_test_workflow_template_transition_field_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        self._test_workflow_template_transition.refresh_from_db()
        self.assertEqual(
            self._test_workflow_template_transition.fields.count(),
            transition_field_count + 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(
            events[0].action_object,
            self._test_workflow_template_transition_field
        )
        self.assertEqual(events[0].target, self._test_workflow_template)
        self.assertEqual(events[0].verb, event_workflow_template_edited.id)

    def test_workflow_template_transition_field_delete_api_view_no_permission(self):
        self._create_test_workflow_template_transition_field()

        transition_field_count = self._test_workflow_template_transition.fields.count()

        self._clear_events()

        response = self._request_test_workflow_template_transition_field_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(
            self._test_workflow_template_transition.fields.count(),
            transition_field_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_transition_field_delete_api_view_with_access(self):
        self._create_test_workflow_template_transition_field()

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        transition_field_count = self._test_workflow_template_transition.fields.count()

        self._clear_events()

        response = self._request_test_workflow_template_transition_field_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        self.assertEqual(
            self._test_workflow_template_transition.fields.count(),
            transition_field_count - 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].target, self._test_workflow_template)
        self.assertEqual(events[0].verb, event_workflow_template_edited.id)

    def test_workflow_template_transition_field_detail_api_view_no_permission(self):
        self._create_test_workflow_template_transition_field()

        self._clear_events()

        response = self._request_test_workflow_template_transition_field_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
        self.assertFalse('results' in response.data)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_transition_field_detail_api_view_with_access(self):
        self._create_test_workflow_template_transition_field()

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_view
        )

        self._clear_events()

        response = self._request_test_workflow_template_transition_field_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data['label'],
            self._test_workflow_template_transition_field.label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_transition_field_edit_via_patch_api_view_no_permission(self):
        self._create_test_workflow_template_transition_field()

        transition_field_label = self._test_workflow_template_transition_field.label

        self._clear_events()

        response = self._request_test_workflow_template_transition_field_edit_via_patch_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self._test_workflow_template_transition_field.refresh_from_db()
        self.assertEqual(
            self._test_workflow_template_transition_field.label, transition_field_label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_transition_field_edit_via_patch_api_view_with_access(self):
        self._create_test_workflow_template_transition_field()

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        transition_field_label = self._test_workflow_template_transition_field.label

        self._clear_events()

        response = self._request_test_workflow_template_transition_field_edit_via_patch_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self._test_workflow_template_transition_field.refresh_from_db()
        self.assertNotEqual(
            self._test_workflow_template_transition_field.label, transition_field_label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(
            events[0].action_object, self._test_workflow_template_transition_field
        )
        self.assertEqual(events[0].target, self._test_workflow_template)
        self.assertEqual(events[0].verb, event_workflow_template_edited.id)

    def test_workflow_template_transition_field_list_api_view_no_permission(self):
        self._create_test_workflow_template_transition_field()

        self._clear_events()

        response = self._request_test_workflow_template_transition_field_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
        self.assertFalse('results' in response.data)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_transition_field_list_api_view_with_access(self):
        self._create_test_workflow_template_transition_field()

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_view
        )

        self._clear_events()

        response = self._request_test_workflow_template_transition_field_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data['results'][0]['label'],
            self._test_workflow_template_transition_field.label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



import json
import logging

from django.db import models
from django.utils.module_loading import import_string
from django.utils.translation import ugettext_lazy as _

from .literals import IMPORT_ERROR_EXCLUSION_TEXTS

logger = logging.getLogger(name=__name__)


class BackendModelMixin(models.Model):
    _backend_model_null_backend = None

    backend_path = models.CharField(
        max_length=128, help_text=_(
            'The dotted Python path to the backend class.'
        ), verbose_name=_('Backend path')
    )
    backend_data = models.TextField(
        blank=True, help_text=_(
            'JSON encoded data for the backend class.'
        ), verbose_name=_('Backend data')
    )

    class Meta:
        abstract = True

    def get_backend(self):
        """
        Retrieves the backend by importing the module and the class.
        """
        try:
            return import_string(dotted_path=self.backend_path)
        except ModuleNotFoundError as exception:
            logger.error(
                'ModuleNotFoundError while importing backend: %s; %s',
                self.backend_path, exception
            )
            if self._backend_model_null_backend:
                return self._backend_model_null_backend
            else:
                raise
        except ImportError as exception:
            logger.error(
                'ImportError while importing backend: %s; %s',
                self.backend_path, exception
            )
            for import_error_exclusion_text in IMPORT_ERROR_EXCLUSION_TEXTS:
                if import_error_exclusion_text in str(exception):
                    raise

            if self._backend_model_null_backend:
                return self._backend_model_null_backend
            else:
                raise

    def get_backend_instance(self):
        return self.get_backend()(
            model_instance_id=self.id, **self.get_backend_data()
        )

    def get_backend_label(self):
        """
        Return the label that the backend itself provides. The backend is
        loaded but not initialized. As such the label returned is a class
        property.
        """
        return self.get_backend().label

    get_backend_label.short_description = _('Backend')
    get_backend_label.help_text = _('The backend class for this entry.')

    def get_backend_data(self):
        return json.loads(s=self.backend_data or '{}')

    def set_backend_data(self, obj):
        self.backend_data = json.dumps(obj=obj)


class ExtraDataModelMixin:
    def __init__(self, *args, **kwargs):
        _instance_extra_data = kwargs.pop('_instance_extra_data', {})
        result = super().__init__(*args, **kwargs)
        for key, value in _instance_extra_data.items():
            setattr(self, key, value)

        return result


class ValueChangeModelMixin:
    @classmethod
    def from_db(cls, db, field_names, values):
        new = super().from_db(db=db, field_names=field_names, values=values)
        new._values_previous = dict(zip(field_names, values))
        return new

    def __init__(self, *args, **kwargs):
        self._values_previous = kwargs
        super().__init__(*args, **kwargs)

    def _get_field_previous_value(self, field):
        return self._values_previous[field]

    def _has_field_changed(self, field):
        if self._state.adding:
            if getattr(self, field) != self._values_previous[field]:
                return True

        return False



from django.utils.translation import ugettext_lazy as _

from mayan.apps.documents.search import (
    document_file_search, document_file_page_search, document_search,
    document_version_search, document_version_page_search
)
from mayan.apps.dynamic_search.classes import SearchModel

from .permissions import permission_metadata_type_view

# Document

document_search.add_model_field(
    field='metadata__metadata_type__name', label=_('Metadata type')
)
document_search.add_model_field(
    field='metadata__value', label=_('Metadata value')
)

# Document file

document_file_search.add_model_field(
    field='document__metadata__metadata_type__name',
    label=_('Document metadata type')
)
document_file_search.add_model_field(
    field='document__metadata__value',
    label=_('Document metadata value')
)

# Document file page

document_file_page_search.add_model_field(
    field='document_file__document__metadata__metadata_type__name',
    label=_('Document metadata type')
)
document_file_page_search.add_model_field(
    field='document_file__document__metadata__value',
    label=_('Document metadata value')
)

# Document version

document_version_search.add_model_field(
    field='document__metadata__metadata_type__name',
    label=_('Document metadata type')
)
document_version_search.add_model_field(
    field='document__metadata__value',
    label=_('Document metadata value')
)

# Document version page

document_version_page_search.add_model_field(
    field='document_version__document__metadata__metadata_type__name',
    label=_('Document metadata type')
)
document_version_page_search.add_model_field(
    field='document_version__document__metadata__value',
    label=_('Document metadata value')
)

# Metadata type

metadata_type_search = SearchModel(
    app_label='metadata', model_name='MetadataType',
    permission=permission_metadata_type_view,
    serializer_path='mayan.apps.metadata.serializers.MetadataTypeSerializer'
)

metadata_type_search.add_model_field(field='default')
metadata_type_search.add_model_field(field='label')
metadata_type_search.add_model_field(field='lookup')
metadata_type_search.add_model_field(field='name')
metadata_type_search.add_model_field(field='parser')
metadata_type_search.add_model_field(field='validation')



from django.utils.translation import ugettext_lazy as _

from mayan.apps.acls.classes import ModelPermission
from mayan.apps.acls.permissions import permission_acl_edit, permission_acl_view
from mayan.apps.common.apps import MayanAppConfig
from mayan.apps.common.menus import (
    menu_multi_item, menu_object, menu_secondary, menu_tools
)
from mayan.apps.events.classes import EventModelRegistry, ModelEventType
from mayan.apps.navigation.classes import SourceColumn

from .events import (
    event_cache_edited, event_cache_partition_purged, event_cache_purged
)
from .links import (
    link_caches_list, link_cache_multiple_purge, link_cache_purge
)
from .permissions import permission_cache_purge, permission_cache_view


class FileCachingConfig(MayanAppConfig):
    app_namespace = 'file_caching'
    app_url = 'file_caching'
    has_tests = True
    name = 'mayan.apps.file_caching'
    verbose_name = _('File caching')

    def ready(self):
        super().ready()

        Cache = self.get_model(model_name='Cache')
        CachePartition = self.get_model(model_name='CachePartition')

        EventModelRegistry.register(model=Cache)
        EventModelRegistry.register(model=CachePartition)

        ModelEventType.register(
            event_types=(event_cache_edited, event_cache_purged,),
            model=Cache
        )
        ModelEventType.register(
            event_types=(event_cache_partition_purged,), model=CachePartition
        )

        ModelPermission.register(
            model=Cache, permissions=(
                permission_acl_edit, permission_acl_view,
                permission_cache_purge, permission_cache_view
            )
        )

        SourceColumn(
            attribute='label', is_identifier=True, is_object_absolute_url=True, source=Cache
        )
        SourceColumn(
            attribute='get_maximum_size_display', include_label=True,
            is_sortable=True, sort_field='maximum_size', source=Cache
        )
        SourceColumn(
            attribute='get_total_size_display', include_label=True,
            source=Cache
        )

        menu_object.bind_links(
            links=(link_cache_purge,),
            sources=(Cache,)
        )
        menu_multi_item.bind_links(
            links=(link_cache_multiple_purge,),
            sources=(Cache,)
        )
        menu_secondary.bind_links(
            links=(link_caches_list,), sources=(
                Cache,
            )
        )

        menu_tools.bind_links(links=(link_caches_list,))



import yaml

from django import forms
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.serialization import yaml_load


class SettingForm(forms.Form):
    value = forms.CharField(
        help_text=_('Enter the new setting value.'), required=False,
        widget=forms.widgets.Textarea()
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setting = self.initial['setting']
        self.fields['value'].help_text = self.setting.help_text
        self.fields['value'].initial = self.setting.serialized_value

    def clean(self):
        try:
            yaml_load(stream=self.cleaned_data['value'])
        except yaml.YAMLError:
            raise ValidationError(
                _(
                    '"%s" not a valid entry.'
                ) % self.cleaned_data['value']
            )
        else:
            self.setting.validate(raw_value=self.cleaned_data['value'])



from django.apps import apps
from django.urls import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from mayan.apps.dashboards.classes import (
    DashboardWidgetList, DashboardWidgetNumeric
)
from mayan.apps.mayan_statistics.icons import icon_statistics

from .icons import (
    icon_dashboard_documents_in_trash, icon_dashboard_document_types,
    icon_dashboard_pages_per_month, icon_dashboard_new_documents_this_month,
    icon_dashboard_total_document,

    icon_document_recently_accessed_list
)
from .links.document_links import (
    link_document_recently_accessed_list,
    link_document_recently_created_list
)
from .links.favorite_links import link_document_favorites_list
from .permissions import (
    permission_document_view, permission_document_type_view
)
from .statistics import (
    new_document_pages_this_month, new_documents_this_month,
)


class DashboardWidgetDocumentFilePagesTotal(DashboardWidgetNumeric):
    icon = icon_dashboard_pages_per_month
    label = _('Total pages')
    link = reverse_lazy(
        viewname='statistics:statistic_detail', kwargs={
            'slug': 'total-document-pages-at-each-month'
        }
    )
    link_icon = icon_statistics

    def get_count(self):
        AccessControlList = apps.get_model(
            app_label='acls', model_name='AccessControlList'
        )
        DocumentFilePage = apps.get_model(
            app_label='documents', model_name='DocumentFilePage'
        )
        return AccessControlList.objects.restrict_queryset(
            permission=permission_document_view, user=self.request.user,
            queryset=DocumentFilePage.valid.all()
        ).count()


class DashboardWidgetDocumentsTotal(DashboardWidgetNumeric):
    icon = icon_dashboard_total_document
    label = _('Total documents')
    link = reverse_lazy(viewname='documents:document_list')

    def get_count(self):
        AccessControlList = apps.get_model(
            app_label='acls', model_name='AccessControlList'
        )
        Document = apps.get_model(
            app_label='documents', model_name='Document'
        )
        return AccessControlList.objects.restrict_queryset(
            permission=permission_document_view, user=self.request.user,
            queryset=Document.valid.all()
        ).count()


class DashboardWidgetDocumentsInTrash(DashboardWidgetNumeric):
    icon = icon_dashboard_documents_in_trash
    label = _('Documents in trash')
    link = reverse_lazy(viewname='documents:document_list_deleted')

    def get_count(self):
        AccessControlList = apps.get_model(
            app_label='acls', model_name='AccessControlList'
        )
        TrashedDocument = apps.get_model(
            app_label='documents', model_name='TrashedDocument'
        )
        return AccessControlList.objects.restrict_queryset(
            permission=permission_document_view, user=self.request.user,
            queryset=TrashedDocument.objects.all()
        ).count()


class DashboardWidgetDocumentsTypesTotal(DashboardWidgetNumeric):
    icon = icon_dashboard_document_types
    label = _('Document types')
    link = reverse_lazy(viewname='documents:document_type_list')

    def get_count(self):
        AccessControlList = apps.get_model(
            app_label='acls', model_name='AccessControlList'
        )
        DocumentType = apps.get_model(
            app_label='documents', model_name='DocumentType'
        )
        return AccessControlList.objects.restrict_queryset(
            permission=permission_document_type_view, user=self.request.user,
            queryset=DocumentType.objects.all()
        ).count()


class DashboardWidgetDocumentsNewThisMonth(DashboardWidgetNumeric):
    icon = icon_dashboard_new_documents_this_month
    label = _('New documents this month')
    link = reverse_lazy(
        viewname='statistics:statistic_detail', kwargs={
            'slug': 'new-documents-per-month'
        }
    )
    link_icon = icon_statistics

    def get_count(self):
        return new_documents_this_month(user=self.request.user)


class DashboardWidgetDocumentsPagesNewThisMonth(DashboardWidgetNumeric):
    icon = icon_dashboard_pages_per_month
    label = _('New pages this month')
    link = reverse_lazy(
        viewname='statistics:statistic_detail', kwargs={
            'slug': 'new-document-pages-per-month'
        }
    )
    link_icon = icon_statistics

    def get_count(self):
        return new_document_pages_this_month(user=self.request.user)


class DashboardWidgetUserRecentlyAccessedDocuments(DashboardWidgetList):
    columns = ('datetime_accessed', 'label',)
    icon = icon_document_recently_accessed_list
    label = link_document_recently_accessed_list.text
    link = reverse_lazy(
        viewname=link_document_recently_accessed_list.view
    )

    def get_object_list(self):
        RecentlyAccessedDocument = apps.get_model(
            app_label='documents', model_name='RecentlyAccessedDocument'
        )

        return RecentlyAccessedDocument.valid.get_for_user(
            user=self.request.user
        )


class DashboardWidgetUserRecentlyCreatedDocuments(DashboardWidgetList):
    columns = ('datetime_created', 'label',)
    icon = link_document_recently_created_list.icon
    label = link_document_recently_created_list.text
    link = reverse_lazy(
        viewname=link_document_recently_created_list.view
    )

    def get_object_list(self):
        AccessControlList = apps.get_model(
            app_label='acls', model_name='AccessControlList'
        )
        RecentlyCreatedDocument = apps.get_model(
            app_label='documents', model_name='RecentlyCreatedDocument'
        )

        queryset = RecentlyCreatedDocument.valid.all()

        return AccessControlList.objects.restrict_queryset(
            permission=permission_document_type_view, user=self.request.user,
            queryset=queryset
        )


class DashboardWidgetUserFavoriteDocuments(DashboardWidgetList):
    columns = ('datetime_added', 'label',)
    icon = link_document_favorites_list.icon
    label = link_document_favorites_list.text
    link = reverse_lazy(
        viewname=link_document_favorites_list.view
    )

    def get_object_list(self):
        FavoriteDocument = apps.get_model(
            app_label='documents', model_name='FavoriteDocument'
        )

        return FavoriteDocument.valid.get_for_user(
            user=self.request.user
        )



from mayan.apps.documents.tests.base import GenericDocumentTestCase
from mayan.apps.documents.tests.literals import TEST_PDF_DOCUMENT_FILENAME

from .literals import (
    TEST_PDF_FILE_METADATA_DOTTED_NAME, TEST_PDF_FILE_METADATA_VALUE
)


class EXIFToolDriverTestCase(GenericDocumentTestCase):
    test_document_filename = TEST_PDF_DOCUMENT_FILENAME

    def test_driver_entries(self):
        self.test_document.submit_for_file_metadata_processing()
        value = self.test_document.get_file_metadata(
            dotted_name=TEST_PDF_FILE_METADATA_DOTTED_NAME
        )
        self.assertEqual(value, TEST_PDF_FILE_METADATA_VALUE)



from mayan.apps.testing.tests.base import GenericViewTestCase

from ..events import (
    event_user_impersonation_ended, event_user_impersonation_started
)
from ..literals import USER_IMPERSONATE_VARIABLE_DISABLE
from ..permissions import permission_users_impersonate

from .mixins import UserImpersonationViewTestMixin


class UserImpersonationViewTestCase(
    UserImpersonationViewTestMixin, GenericViewTestCase
):
    def test_user_impersonate_form_start_view_no_permission(self):
        self._create_test_user()

        self._clear_events()

        response = self._request_test_user_impersonate_form_start_view()
        self.assertEqual(response.status_code, 200)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

        self.expected_content_types = ('application/json',)

        self._clear_events()

        response = self.get(viewname='rest_api:user-current')
        self.assertEqual(response.data['id'], self._test_case_user.pk)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_user_impersonate_form_start_view_with_access(self):
        self._create_test_user()

        self.grant_access(
            obj=self.test_user, permission=permission_users_impersonate
        )

        self._clear_events()

        response = self._request_test_user_impersonate_form_start_view()
        self.assertEqual(response.status_code, 200)

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self.test_user)
        self.assertEqual(events[0].verb, event_user_impersonation_started.id)

        self.expected_content_types = ('application/json',)

        self._clear_events()

        response = self.get(viewname='rest_api:user-current')
        self.assertEqual(response.data['id'], self.test_user.pk)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_user_impersonate_end_view_no_permission(self):
        self._create_test_user()

        self._impersonate_test_user()

        self.grant_access(
            obj=self.test_user, permission=permission_users_impersonate
        )

        self.expected_content_types = ('application/json',)

        self._clear_events()

        response = self.get(viewname='rest_api:user-current')
        self.assertEqual(response.data['id'], self.test_user.pk)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

        self.revoke_access(
            obj=self.test_user, permission=permission_users_impersonate
        )

        self.expected_content_types = ('text/html; charset=utf-8',)

        self._clear_events()

        response = self._request_test_user_impersonate_end_view()
        self.assertEqual(response.status_code, 200)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

        self.expected_content_types = ('application/json',)

        self._clear_events()

        response = self.get(viewname='rest_api:user-current')
        self.assertEqual(response.data['id'], self._test_case_user.pk)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_user_impersonate_end_view_with_access(self):
        self._create_test_user()

        self._impersonate_test_user()

        self.grant_access(
            obj=self.test_user, permission=permission_users_impersonate
        )

        self.expected_content_types = ('application/json',)

        self._clear_events()

        response = self.get(viewname='rest_api:user-current')
        self.assertEqual(response.data['id'], self.test_user.pk)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

        self.expected_content_types = ('text/html; charset=utf-8',)

        self._clear_events()

        response = self._request_test_user_impersonate_end_view()
        self.assertEqual(response.status_code, 200)

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self.test_user)
        self.assertEqual(events[0].verb, event_user_impersonation_ended.id)

        self.expected_content_types = ('application/json',)

        self._clear_events()

        response = self.get(viewname='rest_api:user-current')
        self.assertEqual(response.data['id'], self._test_case_user.pk)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_session_impersonate_set_with_acess(self):
        self._create_test_user()

        self.grant_access(
            obj=self.test_user, permission=permission_users_impersonate
        )

        self._impersonate_test_user()

        self.expected_content_types = ('application/json',)

        self._clear_events()

        response = self.get(viewname='rest_api:user-current')
        self.assertEqual(response.data['id'], self.test_user.pk)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_session_impersonate_unset_with_access(self):
        self._create_test_user()
        self.grant_access(
            obj=self.test_user, permission=permission_users_impersonate
        )

        self._impersonate_test_user()

        self.expected_content_types = ('application/json',)

        self._clear_events()

        response = self.get(
            viewname='rest_api:user-current', query={
                USER_IMPERSONATE_VARIABLE_DISABLE: ''
            }
        )
        self.assertEqual(response.data['id'], self._test_case_user.pk)

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self.test_user)
        self.assertEqual(events[0].verb, event_user_impersonation_ended.id)

    def test_user_impersonate_start_view_no_permission(self):
        self._create_test_user()

        self._clear_events()

        response = self._request_test_user_impersonate_start_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

        self.expected_content_types = ('application/json',)

        self._clear_events()

        response = self.get(viewname='rest_api:user-current')
        self.assertEqual(response.data['id'], self._test_case_user.pk)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_user_impersonate_start_view_with_access(self):
        self._create_test_user()

        self.grant_access(
            obj=self.test_user, permission=permission_users_impersonate
        )

        self._clear_events()

        response = self._request_test_user_impersonate_start_view()
        self.assertEqual(response.status_code, 200)

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self.test_user)
        self.assertEqual(events[0].verb, event_user_impersonation_started.id)

        self.expected_content_types = ('application/json',)

        self._clear_events()

        response = self.get(viewname='rest_api:user-current')
        self.assertEqual(response.data['id'], self.test_user.pk)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



from ..classes import Worker, CeleryQueue

from .literals import TEST_QUEUE_LABEL, TEST_QUEUE_NAME, TEST_WORKER_NAME


class TaskManagerTestMixin:
    def setUp(self):
        super().setUp()
        self._test_queues = []

    def tearDown(self):
        for test_queue in self._test_queues:
            test_queue.remove()

    def _create_test_queue(self, label=None, name=None):
        self.test_worker = Worker(name=TEST_WORKER_NAME)

        total_test_queues = len(self._test_queues)
        label = label or '{}_{}'.format(TEST_QUEUE_LABEL, total_test_queues)
        name = name or '{}_{}'.format(TEST_QUEUE_NAME, total_test_queues)

        self._test_queue = CeleryQueue(
            label=label, name=name, worker=self.test_worker
        )
        self._test_queues.append(self._test_queue)


class TaskManagerViewTestMixin:
    def _request_queue_list(self):
        return self.get(
            viewname='task_manager:queue_list', follow=True
        )



from mayan.apps.testing.tests.base import BaseTestCase

from .mixins.workflow_template_mixins import WorkflowTemplateTestMixin


class WorkflowModelTestCase(WorkflowTemplateTestMixin, BaseTestCase):
    def test_workflow_template_preview(self):
        self._create_test_workflow_template()
        self.assertTrue(self._test_workflow_template.get_api_image_url())



from mayan.apps.document_states.permissions import permission_workflow_template_edit
from mayan.apps.document_states.tests.base import ActionTestCase
from mayan.apps.document_states.tests.mixins.workflow_template_mixins import WorkflowTemplateTestMixin
from mayan.apps.document_states.tests.mixins.workflow_template_state_mixins import WorkflowTemplateStateActionViewTestMixin

from mayan.apps.testing.tests.base import GenericViewTestCase

from ..models import Cabinet
from ..workflow_actions import CabinetAddAction, CabinetRemoveAction

from .mixins import CabinetTestMixin


class CabinetWorkflowActionTestCase(CabinetTestMixin, ActionTestCase):
    def setUp(self):
        super().setUp()
        self._create_test_cabinet()

    def test_cabinet_add_action(self):
        action = CabinetAddAction(
            form_data={'cabinets': Cabinet.objects.all()}
        )
        action.execute(context={'document': self.test_document})

        self.assertTrue(
            self.test_document in self.test_cabinet.documents.all()
        )

    def test_cabinet_remove_action(self):
        self.test_cabinet.document_add(document=self.test_document)

        action = CabinetRemoveAction(
            form_data={'cabinets': Cabinet.objects.all()}
        )
        action.execute(context={'document': self.test_document})

        self.assertFalse(
            self.test_document in self.test_cabinet.documents.all()
        )


class CabinetWorkflowActionViewTestCase(
    CabinetTestMixin, WorkflowTemplateStateActionViewTestMixin,
    WorkflowTemplateTestMixin, GenericViewTestCase
):
    def test_cabinet_add_action_create_get_view(self):
        self._create_test_workflow_template()
        self._create_test_workflow_template_state()
        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        response = self._request_test_workflow_template_state_action_create_get_view(
            class_path='mayan.apps.cabinets.workflow_actions.CabinetAddAction'
        )
        self.assertEqual(response.status_code, 200)

    def test_cabinet_remove_action_create_get_view(self):
        self._create_test_workflow_template()
        self._create_test_workflow_template_state()
        self._create_test_cabinet()
        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        response = self._request_test_workflow_template_state_action_create_get_view(
            class_path='mayan.apps.cabinets.workflow_actions.CabinetRemoveAction'
        )
        self.assertEqual(response.status_code, 200)



from django.db import models, migrations, reset_queries
from django.db.models import Max
from django.db.models.functions import Concat


def operation_document_file_filename_copy(apps, schema_editor):
    cursor_main = schema_editor.connection.cursor()
    cursor_document_file = schema_editor.connection.cursor()

    query_document_file = '''
        UPDATE {documents_documentfile} SET {filename} = %s WHERE {documents_documentfile}.{id} = %s;
    '''.format(
        documents_documentfile=schema_editor.connection.ops.quote_name(
            name='documents_documentfile'
        ),
        filename=schema_editor.connection.ops.quote_name(name='filename'),
        id=schema_editor.connection.ops.quote_name(name='id')
    )

    query = '''
        SELECT
            {documents_document}.{label},
            {documents_documentfile}.{id}
        FROM {documents_documentfile}
        INNER JOIN {documents_document} ON (
            {documents_documentfile}.{document_id} = {documents_document}.{id}
        )
    '''.format(
        document_id=schema_editor.connection.ops.quote_name(
            name='document_id'
        ),
        documents_document=schema_editor.connection.ops.quote_name(
            name='documents_document'
        ),
        documents_documentfile=schema_editor.connection.ops.quote_name(
            name='documents_documentfile'
        ),
        id=schema_editor.connection.ops.quote_name(name='id'),
        label=schema_editor.connection.ops.quote_name(name='label')
    )

    cursor_main.execute(query)

    for row in cursor_main.fetchall():
        cursor_document_file.execute(
            query_document_file, row
        )


def operation_set_active_versions(apps, schema_editor):
    Document = apps.get_model(
        app_label='documents', model_name='Document'
    )
    DocumentVersion = apps.get_model(
        app_label='documents', model_name='DocumentVersion'
    )

    cursor_main = schema_editor.connection.create_cursor(name='cursor_main')
    cursor_document_version = schema_editor.connection.cursor()

    # Select the latest version for each document by date.
    document_queryset = Document.objects.only('id').annotate(
        latest_version_timestamp=Max('versions__timestamp')
    )

    # Exclude documents with no latest version.
    document_queryset = document_queryset.exclude(
        latest_version_timestamp=None
    )

    # Create a new unique version identifier.
    document_queryset = document_queryset.annotate(
        version_identifier=Concat(
            'id', 'latest_version_timestamp', output_field=models.CharField()
        )
    )

    # Get all document versions and add a new unique version identifier.
    document_version_queryset = DocumentVersion.objects.only('id').annotate(
        version_identifier=Concat(
            'document_id', 'timestamp', output_field=models.CharField()
        )
    )

    # Set all version as not active.
    DocumentVersion.objects.update(active=False)

    # Workaround MySQL: (1093, "You can't specify target table 'documents_documentversion' for update in FROM clause")
    document_version_queryset = document_version_queryset.filter(
        version_identifier__in=document_queryset.values('version_identifier')
    ).values('id')

    compiler = document_version_queryset.query.get_compiler(
        connection=schema_editor.connection
    )

    cursor_main.execute(
        *document_version_queryset.query.as_sql(
            compiler=compiler, connection=schema_editor.connection
        )
    )

    FETCH_SIZE = 10000
    query_document_version_active_update = '''
        UPDATE {documents_documentversion} SET {active} = '1' WHERE {documents_documentversion}.{id} IN {{}};
    '''.format(
        documents_documentversion=schema_editor.connection.ops.quote_name(
            name='documents_documentversion'
        ),
        active=schema_editor.connection.ops.quote_name(name='active'),
        id=schema_editor.connection.ops.quote_name(name='id')
    )

    while True:
        rows = cursor_main.fetchmany(FETCH_SIZE)
        document_version_values = []

        if not rows:
            break

        for row in rows:
            document_version_values += (row[0],)

        query_argument_placeholders = ('%s',) * len(rows)
        values_query = '({})'.format(', '.join(query_argument_placeholders))
        cursor_document_version.execute(
            query_document_version_active_update.format(values_query), document_version_values
        )
        reset_queries()


class Migration(migrations.Migration):
    dependencies = [
        ('documents', '0066_documentfile_filename'),
    ]

    operations = [
        migrations.RunPython(
            code=operation_document_file_filename_copy,
            reverse_code=migrations.RunPython.noop
        ),
        migrations.RunPython(
            code=operation_set_active_versions,
            reverse_code=migrations.RunPython.noop
        ),
    ]



from django.apps import apps
from django.core.exceptions import PermissionDenied
from django.template.loader import render_to_string

from mayan.apps.navigation.html_widgets import SourceColumnWidget

from .permissions import permission_tag_view


class DocumentTagWidget(SourceColumnWidget):
    """
    A tag widget that displays the tags for the given document.
    """
    template_name = 'tags/document_tags_widget.html'

    def get_extra_context(self):
        AccessControlList = apps.get_model(
            app_label='acls', model_name='AccessControlList'
        )

        try:
            AccessControlList.objects.check_access(
                obj=self.value,
                permissions=(permission_tag_view,),
                user=self.request.user
            )
        except PermissionDenied:
            queryset = self.value.tags.none()
        else:
            queryset = self.value.get_tags(
                permission=permission_tag_view,
                user=self.request.user
            )

        return {'tags': queryset}


def widget_single_tag(tag):
    return render_to_string(
        template_name='tags/tag_widget.html', context={'tag': tag}
    )



class RecentlyCreatedDocumentAPIViewTestMixin:
    def _request_recently_created_document_list_api_view(self):
        return self.get(
            viewname='rest_api:recentlycreateddocument-list'
        )


class RecentlyCreatedDocumentViewTestMixin:
    def _request_test_recently_created_document_list_view(self):
        return self.get(viewname='documents:document_recently_created_list')



from furl import furl

from django.conf import settings
from django.contrib.auth import authenticate
from django.test import override_settings
from django.urls import reverse
from django.utils.http import urlunquote_plus

from mayan.apps.smart_settings.classes import SettingNamespace
from mayan.apps.testing.tests.base import GenericViewTestCase

from ..classes import AuthenticationBackend

from .literals import (
    PATH_AUTHENTICATION_BACKEND_EMAIL, PATH_AUTHENTICATION_BACKEND_USERNAME
)
from .mixins import LoginViewTestMixin


class AuthenticationBackendTestCase(LoginViewTestMixin, GenericViewTestCase):
    authenticated_url = reverse(viewname='common:home')
    authentication_url = urlunquote_plus(
        furl(
            path=reverse(settings.LOGIN_URL), args={
                'next': authenticated_url
            }
        ).tostr()
    )
    auto_login_user = False
    create_test_case_superuser = True

    def setUp(self):
        super().setUp()
        SettingNamespace.invalidate_cache_all()

    @override_settings(AUTHENTICATION_BACKEND=PATH_AUTHENTICATION_BACKEND_EMAIL)
    def test_email_authentication_backend(self):
        AuthenticationBackend.cls_initialize()

        user = authenticate(
            username=self._test_case_superuser.email,
            password=self._test_case_superuser.cleartext_password
        )
        self.assertEqual(user, self._test_case_superuser)

    @override_settings(AUTHENTICATION_BACKEND=PATH_AUTHENTICATION_BACKEND_USERNAME)
    def test_username_authentication_backend(self):
        AuthenticationBackend.cls_initialize()

        user = authenticate(
            username=self._test_case_superuser.username,
            password=self._test_case_superuser.cleartext_password
        )
        self.assertEqual(user, self._test_case_superuser)



from django.conf import settings

from django.utils.translation import ugettext_lazy as _

import mayan

DEFAULT_COMMON_COLLAPSE_LIST_MENU_LIST_FACET = False
DEFAULT_COMMON_COLLAPSE_LIST_MENU_OBJECT = False
DEFAULT_COMMON_DISABLED_APPS = settings.COMMON_DISABLED_APPS
DEFAULT_COMMON_EXTRA_APPS = settings.COMMON_EXTRA_APPS
DEFAULT_COMMON_EXTRA_APPS_PRE = settings.COMMON_EXTRA_APPS_PRE
DEFAULT_COMMON_HOME_VIEW = 'common:home'
DEFAULT_COMMON_HOME_VIEW_DASHBOARD_NAME = 'user'
DEFAULT_COMMON_PROJECT_TITLE = mayan.__title__
DEFAULT_COMMON_PROJECT_URL = mayan.__website__

EMPTY_LABEL = '---------'

MESSAGE_DEPRECATION_WARNING = _(
    'This feature has been deprecated and will be removed in a future version.'
)

TIME_DELTA_UNIT_DAYS = 'days'
TIME_DELTA_UNIT_HOURS = 'hours'
TIME_DELTA_UNIT_MINUTES = 'minutes'

TIME_DELTA_UNIT_CHOICES = (
    (TIME_DELTA_UNIT_DAYS, _('Days')),
    (TIME_DELTA_UNIT_HOURS, _('Hours')),
    (TIME_DELTA_UNIT_MINUTES, _('Minutes')),
)



from django.utils.translation import ugettext_lazy as _

from mayan.apps.authentication.link_conditions import condition_user_is_authenticated
from mayan.apps.authentication.utils import get_context_user
from mayan.apps.navigation.classes import Link

from ..icons import (
    icon_favorite_document_add, icon_favorite_document_list,
    icon_favorite_document_remove
)
from ..permissions import permission_document_view


def condition_is_in_favorites(context, resolved_object):
    if condition_user_is_authenticated(context=context, resolved_object=resolved_object):
        user = get_context_user(context=context)
        return resolved_object.favorites.filter(user=user).exists()


def condition_not_is_in_favorites(context, resolved_object):
    if condition_user_is_authenticated(context=context, resolved_object=resolved_object):
        user = get_context_user(context=context)
        return not resolved_object.favorites.filter(user=user).exists()


link_document_favorites_list = Link(
    condition=condition_user_is_authenticated,
    icon=icon_favorite_document_list, text=_('Favorites'),
    view='documents:document_favorite_list'
)
link_document_favorites_add = Link(
    condition=condition_not_is_in_favorites,
    args='resolved_object.id', icon=icon_favorite_document_add,
    permissions=(permission_document_view,), text=_('Add to favorites'),
    view='documents:document_favorite_add'
)
link_document_favorites_remove = Link(
    condition=condition_is_in_favorites,
    args='resolved_object.id', icon=icon_favorite_document_remove,
    permissions=(permission_document_view,), text=_('Remove from favorites'),
    view='documents:document_favorite_remove'
)
link_document_favorites_add_multiple = Link(
    text=_('Add to favorites'), icon=icon_favorite_document_add,
    view='documents:document_favorite_add_multiple'
)
link_document_favorites_remove_multiple = Link(
    text=_('Remove from favorites'), icon=icon_favorite_document_remove,
    view='documents:document_favorite_remove_multiple'
)



from io import StringIO

from django.contrib.auth import get_user_model
from django.core import management

from mayan.apps.testing.tests.base import BaseTestCase
from mayan.apps.testing.tests.utils import mute_stdout

from ..events import event_otp_disabled
from ..models import UserOTPData

from .mixins import AuthenticationOTPTestMixin


class AuthenticationOTPDisableManagementCommandTestCase(
    AuthenticationOTPTestMixin, BaseTestCase
):
    create_test_case_superuser = True

    def _call_command(self):
        with mute_stdout():
            management.call_command(
                'authentication_otp_disable',
                self._test_case_superuser.username
            )

    def test_command(self):
        self._enable_test_otp()

        self._clear_events()

        self._call_command()

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_superuser)
        self.assertEqual(events[0].target, self._test_case_superuser)
        self.assertEqual(events[0].verb, event_otp_disabled.id)


class AuthenticationOTPInitializeManagementCommandTestCase(
    AuthenticationOTPTestMixin, BaseTestCase
):
    create_test_case_superuser = True

    def _call_command(self):
        with mute_stdout():
            management.call_command(
                'authentication_otp_initialize'
            )

    def test_command(self):
        test_user_count = get_user_model().objects.count()

        UserOTPData.objects.all().delete()

        self._clear_events()

        self._call_command()

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

        self.assertEqual(
            UserOTPData.objects.count(), test_user_count
        )


class AuthenticationOTPStatusManagementCommandTestCase(
    AuthenticationOTPTestMixin, BaseTestCase
):
    create_test_case_superuser = True

    def _call_command(self):
        output = StringIO()
        options = {
            'stdout': output
        }

        management.call_command(
            'authentication_otp_status', self._test_case_superuser.username,
            **options
        )
        output.seek(0)

        return output.getvalue()

    def test_command_with_otp_disabled(self):
        self._clear_events()

        output = self._call_command()
        self.assertTrue('disabled' in output)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_command_with_otp_enabled(self):
        self._enable_test_otp()

        self._clear_events()

        output = self._call_command()
        self.assertTrue('enabled' in output)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('documents', '0058_auto_20200916_1146'),
    ]
    operations = [
        migrations.DeleteModel(
            name='DocumentPageResult',
        ),
        migrations.RenameModel(
            old_name='DocumentPage',
            new_name='DocumentFilePage',
        ),
        migrations.CreateModel(
            name='DocumentFilePageResult',
            fields=[
            ],
            options={
                'verbose_name': 'Document file page',
                'verbose_name_plural': 'Document file pages',
                'ordering': ('document_file__document', 'page_number'),
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('documents.documentfilepage',),
        ),
        migrations.AlterModelOptions(
            name='documentfilepage',
            options={
                'ordering': ('page_number',),
                'verbose_name': 'Document file page',
                'verbose_name_plural': 'Document file pages'
            },
        ),
    ]



from .classes import Layer


class LayerViewMixin:
    def dispatch(self, request, *args, **kwargs):
        self.layer = self.get_layer()
        return super().dispatch(request=request, *args, **kwargs)

    def get_layer(self):
        return Layer.get(
            name=self.kwargs['layer_name']
        )



from django.apps import apps
from django.utils.translation import ugettext_lazy as _

from mayan.apps.authentication.link_conditions import (
    condition_is_current_user, condition_not_is_current_user,
    condition_user_is_authenticated
)
from mayan.apps.navigation.classes import Link
from mayan.apps.navigation.utils import get_content_type_kwargs_factory

from .icons import (
    icon_event_types_subscriptions_list,
    icon_events_for_object, icon_events_for_object_clear,
    icon_events_for_object_export, icon_events_list, icon_events_list_clear,
    icon_events_list_export, icon_notification_mark_read,
    icon_notification_mark_read_all,
    icon_object_event_types_user_subscriptions_list,
    icon_user_notifications_list
)
from .permissions import (
    permission_events_clear, permission_events_export, permission_events_view
)


def condition_can_be_cleared(context, resolved_object):
    # Hidden import.
    from mayan.apps.acls.classes import ModelPermission
    return permission_events_clear.stored_permission in ModelPermission.get_for_instance(
        instance=context['resolved_object']
    )


def condition_can_be_exported(context, resolved_object):
    # Hidden import.
    from mayan.apps.acls.classes import ModelPermission
    return permission_events_export.stored_permission in ModelPermission.get_for_instance(
        instance=context['resolved_object']
    )


def get_unread_notification_count(context):
    Notification = apps.get_model(
        app_label='events', model_name='Notification'
    )
    if context.request.user.is_authenticated:
        return Notification.objects.filter(
            user=context.request.user
        ).filter(read=False).count()


link_events_for_object = Link(
    icon=icon_events_for_object,
    kwargs=get_content_type_kwargs_factory(variable_name='resolved_object'),
    permissions=(permission_events_view,), text=_('Events'),
    view='events:events_for_object'
)
link_events_for_object_clear = Link(
    condition=condition_can_be_cleared,
    icon=icon_events_for_object_clear,
    kwargs=get_content_type_kwargs_factory(variable_name='resolved_object'),
    permissions=(permission_events_view,), text=_('Clear events'),
    view='events:events_for_object_clear'
)
link_events_for_object_export = Link(
    condition=condition_can_be_exported,
    icon=icon_events_for_object_export,
    kwargs=get_content_type_kwargs_factory(variable_name='resolved_object'),
    permissions=(permission_events_view,), text=_('Export events'),
    view='events:events_for_object_export'
)
link_events_list = Link(
    icon=icon_events_list, text=_('Events'), view='events:events_list'
)
link_events_list_clear = Link(
    icon=icon_events_list_clear, text=_('Clear events'),
    view='events:events_list_clear'
)
link_events_list_export = Link(
    icon=icon_events_list_export, text=_('Export events'),
    view='events:events_list_export'
)
link_event_types_subscriptions_list = Link(
    condition=condition_is_current_user,
    icon=icon_event_types_subscriptions_list,
    text=_('Event subscriptions'),
    view='events:event_types_user_subscriptions_list'
)
link_notification_mark_read = Link(
    args='object.pk', icon=icon_notification_mark_read,
    text=_('Mark as seen'), view='events:notification_mark_read'
)
link_notification_mark_read_all = Link(
    icon=icon_notification_mark_read_all, text=_('Mark all as seen'),
    view='events:notification_mark_read_all'
)
link_object_event_types_user_subscriptions_list = Link(
    condition=condition_not_is_current_user,
    icon=icon_object_event_types_user_subscriptions_list,
    kwargs=get_content_type_kwargs_factory(variable_name='resolved_object'),
    permissions=(permission_events_view,), text=_('Subscriptions'),
    view='events:object_event_types_user_subscriptions_list'
)
link_user_notifications_list = Link(
    badge_text=get_unread_notification_count,
    condition=condition_user_is_authenticated,
    icon=icon_user_notifications_list, text='',
    view='events:user_notifications_list'
)



from django.test import override_settings

from mayan.apps.documents.tests.base import GenericDocumentTestCase
from mayan.apps.documents.tests.literals import TEST_DEU_DOCUMENT_PATH

from .literals import (
    TEST_DOCUMENT_VERSION_OCR_CONTENT, TEST_DOCUMENT_VERSION_OCR_CONTENT_DEU_1,
    TEST_DOCUMENT_VERSION_OCR_CONTENT_DEU_2
)


@override_settings(OCR_AUTO_OCR=True)
class DocumentOCRTestCase(GenericDocumentTestCase):
    def test_ocr_language_backends_eng(self):
        content = self.test_document_version.pages.first().ocr_content.content
        self.assertTrue(TEST_DOCUMENT_VERSION_OCR_CONTENT in content)


@override_settings(OCR_AUTO_OCR=True)
class GermanOCRSupportTestCase(GenericDocumentTestCase):
    test_document_language = 'deu'
    test_document_path = TEST_DEU_DOCUMENT_PATH

    def test_ocr_language_backends_end(self):
        content = self.test_document.version_active.pages.first().ocr_content.content

        self.assertTrue(
            TEST_DOCUMENT_VERSION_OCR_CONTENT_DEU_1 in content
        )
        self.assertTrue(
            TEST_DOCUMENT_VERSION_OCR_CONTENT_DEU_2 in content
        )



from django.core import management

from ...classes import Setting


class Command(management.BaseCommand):
    help = 'Display the current settings.'

    def add_arguments(self, parser):
        parser.add_argument(
            dest='filter_term', nargs='?', help='Use this term to filter the '
            'list of settings.'
        )

        parser.add_argument(
            '--namespace', action='store', dest='namespace',
            help='Name (not label) of the namespace for which to filter the '
            'list of settings. Names are lowercase with words separated by '
            'underscore.'
        )

    def handle(self, *args, **options):
        self.stdout.write(
            msg=Setting.dump_data(
                namespace=options.get('namespace'),
                filter_term=options.get('filter_term')
            )
        )



from PIL import Image
import qrcode
from qrcode.image.styledpil import StyledPilImage
from qrcode.image.styles.moduledrawers import CircleModuleDrawer

from django import forms
from django.contrib.staticfiles.storage import staticfiles_storage

from .widgets import AssetImageWidget, Base64ImageWidget

LOGO_PATH = 'converter/images/logo.png'


class AssetImageField(forms.fields.Field):
    widget = AssetImageWidget


class ReadOnlyImageField(forms.ImageField):
    def clean(self, data, initial=None):
        return ''


class QRCodeImageField(ReadOnlyImageField):
    widget = Base64ImageWidget

    def prepare_value(self, value):
        instance = qrcode.QRCode()
        instance.add_data(value)
        instance.make(fit=True)

        with staticfiles_storage.open(name=LOGO_PATH, mode='rb') as file_object:
            embeded_image = Image.open(fp=file_object)

            qrcode_image = instance.make_image(
                image_factory=StyledPilImage,
                module_drawer=CircleModuleDrawer(),
                embeded_image=embeded_image
            )

            size = qrcode_image.height / 2

            self.widget.attrs['height'] = size
            self.widget.attrs['width'] = size

            return qrcode_image



from django.db import models, migrations


class Migration(migrations.Migration):
    dependencies = [
        ('documents', '0027_auto_20150824_0702'),
    ]

    operations = [
        migrations.CreateModel(
            name='NewVersionBlock',
            fields=[
                (
                    'id', models.AutoField(
                        auto_created=True, primary_key=True, serialize=False,
                        verbose_name='ID'
                    )
                ),
                (
                    'document', models.ForeignKey(
                        on_delete=models.CASCADE, to='documents.Document',
                        verbose_name='Document'
                    )
                ),
            ],
            options={
                'verbose_name': 'New version block',
                'verbose_name_plural': 'New version blocks',
            },
            bases=(models.Model,),
        ),
    ]



from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.apps import MayanAppConfig
from mayan.apps.common.menus import (
    menu_about, menu_list_facet, menu_secondary, menu_tools
)
from mayan.apps.navigation.classes import SourceColumn
from mayan.apps.views.html_widgets import TwoStateWidget

from .classes import Dependency, DependencyGroup, DependencyGroupEntry
from .links import (
    link_check_version, link_dependency_group_entry_detail,
    link_dependency_group_entry_list, link_dependency_group_list,
    link_packages_licenses, link_dependency_tool
)


class DependenciesApp(MayanAppConfig):
    app_namespace = 'dependencies'
    app_url = 'dependencies'
    has_rest_api = False
    has_tests = True
    name = 'mayan.apps.dependencies'
    verbose_name = _('Dependencies')

    def ready(self):
        super().ready()

        Dependency.load_modules()

        SourceColumn(
            attribute='get_label', is_identifier=True, label=_('Label'),
            order=-1, source=Dependency
        )
        SourceColumn(
            attribute='name', include_label=True, label=_('Internal name'),
            order=0, source=Dependency
        )
        SourceColumn(
            attribute='get_help_text', include_label=True,
            label=_('Description'), order=1, source=Dependency
        )
        SourceColumn(
            attribute='class_name_verbose_name', include_label=True,
            label=_('Type'), order=2, source=Dependency
        )
        SourceColumn(
            attribute='get_other_data', include_label=True,
            label=_('Other data'), order=3, source=Dependency
        )
        SourceColumn(
            attribute='app_label_verbose_name', include_label=True,
            label=_('Declared by'), order=4, source=Dependency
        )
        SourceColumn(
            attribute='get_version_string', include_label=True,
            label=_('Version'), order=5, source=Dependency
        )
        SourceColumn(
            attribute='get_environments_verbose_name', include_label=True,
            label=_('Environment'), order=6, source=Dependency
        )
        SourceColumn(
            attribute='check', include_label=True, label=_('Check'), order=7,
            source=Dependency, widget=TwoStateWidget
        )

        SourceColumn(
            attribute='label', is_identifier=True, label=_('Label'),
            order=0, source=DependencyGroup
        )
        SourceColumn(
            attribute='help_text', include_label=True,
            label=_('Description'), order=1, source=DependencyGroup
        )

        SourceColumn(
            attribute='label', is_identifier=True, label=_('Label'), order=0,
            source=DependencyGroupEntry
        )
        SourceColumn(
            attribute='help_text', include_label=True,
            label=_('Description'), order=1, source=DependencyGroupEntry
        )

        # Position #7 which is after "License" link.
        menu_about.bind_links(
            links=(link_packages_licenses,), position=7
        )

        menu_list_facet.bind_links(
            links=(link_dependency_group_entry_list,),
            sources=(DependencyGroup,)
        )
        menu_list_facet.bind_links(
            links=(link_dependency_group_entry_detail,),
            sources=(DependencyGroupEntry,)
        )

        menu_secondary.bind_links(
            links=(link_dependency_group_list,),
            sources=(
                DependencyGroup,
                'dependencies:dependency_group_list',
            )
        )

        menu_tools.bind_links(
            links=(link_dependency_tool, link_check_version)
        )



from django.contrib import messages
from django.utils.translation import ugettext_lazy as _

from actstream.models import Action

from mayan.apps.common.classes import QuerysetParametersSerializer
from mayan.apps.views.generics import ConfirmView
from mayan.apps.views.mixins import ExternalContentTypeObjectViewMixin

from ..permissions import permission_events_export
from ..tasks import task_event_queryset_export

from .mixins import EventViewMixin

__all__ = (
    'EventListExportView', 'ObjectEventExportView', 'VerbEventExportView'
)


class EventExportBaseView(ConfirmView):
    object_permission = permission_events_export

    def get_extra_context(self):
        return {
            'message': _(
                'The process will be performed in the background. '
                'The exported events will be available in the downloads area.'
            )
        }

    def view_action(self):
        decomposed_queryset = QuerysetParametersSerializer.decompose(
            _model=Action, **self.get_queryset_parameters()
        )

        task_event_queryset_export.apply_async(
            kwargs={
                'decomposed_queryset': decomposed_queryset,
                'user_id': self.request.user.pk
            }
        )

        messages.success(
            request=self.request, message=_(
                'Event list export task queued successfully.'
            )
        )


class EventListExportView(EventExportBaseView):
    object_permission = permission_events_export

    def get_extra_context(self):
        context = super().get_extra_context()
        context.update(
            {
                'title': _('Export events'),
            }
        )
        return context

    def get_queryset_parameters(self):
        return {
            '_method_name': 'all'
        }


class ObjectEventExportView(
    ExternalContentTypeObjectViewMixin, EventExportBaseView
):
    def get_extra_context(self):
        context = super().get_extra_context()
        context.update(
            {
                'object': self.external_object,
                'title': _('Export events of: %s') % self.external_object,
            }
        )
        return context

    def get_queryset_parameters(self):
        return {
            '_method_name': 'any', 'obj': self.external_object
        }


class VerbEventExportView(EventViewMixin, EventExportBaseView):
    def get_extra_context(self):
        context = super().get_extra_context()
        context.update(
            {
                'title': _(
                    'Export events of type: %s'
                ) % self.get_event_type(id=self.kwargs['verb'])
            }
        )
        return context

    def get_queryset_parameters(self):
        return {
            '_method_name': 'filter', 'verb': self.kwargs['verb']
        }



from django.conf.urls import url

from .api_views import (
    APIDocumentMetadataListView, APIDocumentMetadataView,
    APIDocumentTypeMetadataTypeListView, APIDocumentTypeMetadataTypeView,
    APIMetadataTypeListView, APIMetadataTypeView
)
from .views.document_views import (
    DocumentMetadataAddView, DocumentMetadataEditView,
    DocumentMetadataListView, DocumentMetadataRemoveView
)
from .views.metadata_type_views import (
    MetadataTypeCreateView, MetadataTypeDeleteView, MetadataTypeEditView,
    MetadataTypeListView, DocumentTypeMetadataTypeRelationshipView,
    MetadataTypesDocumentTypeRelationshipView
)


urlpatterns_document_type = [
    url(
        regex=r'^document_types/(?P<document_type_id>\d+)/metadata_types/$',
        name='document_type_metadata_type_relationship',
        view=DocumentTypeMetadataTypeRelationshipView.as_view()
    )
]

urlpatterns_document_metadata = [
    url(
        regex=r'^document_types/(?P<document_type_id>\d+)/metadata_types/$',
        name='document_type_metadata_type_relationship',
        view=DocumentTypeMetadataTypeRelationshipView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/metadata/edit/$',
        name='metadata_edit', view=DocumentMetadataEditView.as_view()
    ),
    url(
        regex=r'^documents/multiple/metadata/edit/$',
        name='metadata_multiple_edit', view=DocumentMetadataEditView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/metadata/view/$',
        name='metadata_view', view=DocumentMetadataListView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/metadata/add/$',
        name='metadata_add', view=DocumentMetadataAddView.as_view()
    ),
    url(
        regex=r'^documents/multiple/metadata/add/$',
        name='metadata_multiple_add', view=DocumentMetadataAddView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/metadata/remove/$',
        name='metadata_remove', view=DocumentMetadataRemoveView.as_view()
    ),
    url(
        regex=r'^documents/multiple/metadata/remove/$',
        name='metadata_multiple_remove',
        view=DocumentMetadataRemoveView.as_view()
    )
]

urlpatterns_metadata_type = [
    url(
        regex=r'^metadata_types/$', name='metadata_type_list',
        view=MetadataTypeListView.as_view()
    ),
    url(
        regex=r'^metadata_types/create/$', name='metadata_type_create',
        view=MetadataTypeCreateView.as_view()
    ),
    url(
        regex=r'^metadata_types/(?P<metadata_type_id>\d+)/delete/$',
        name='metadata_type_delete_single',
        view=MetadataTypeDeleteView.as_view()
    ),
    url(
        regex=r'^metadata_types/(?P<metadata_type_id>\d+)/document_types/$',
        name='metadata_type_document_type_relationship',
        view=MetadataTypesDocumentTypeRelationshipView.as_view()
    ),
    url(
        regex=r'^metadata_types/(?P<metadata_type_id>\d+)/edit/$',
        name='metadata_type_edit', view=MetadataTypeEditView.as_view()
    ),
    url(
        regex=r'^metadata_types/multiple/delete/$',
        name='metadata_type_delete_multiple',
        view=MetadataTypeDeleteView.as_view()
    )
]

urlpatterns = []
urlpatterns.extend(urlpatterns_document_metadata)
urlpatterns.extend(urlpatterns_document_type)
urlpatterns.extend(urlpatterns_metadata_type)

api_urls = [
    url(
        regex=r'^metadata_types/$', name='metadatatype-list',
        view=APIMetadataTypeListView.as_view()
    ),
    url(
        regex=r'^metadata_types/(?P<metadata_type_id>\d+)/$',
        name='metadatatype-detail', view=APIMetadataTypeView.as_view()
    ),
    url(
        regex=r'^document_types/(?P<document_type_id>\d+)/metadata_types/$',
        name='documenttypemetadatatype-list',
        view=APIDocumentTypeMetadataTypeListView.as_view()
    ),
    url(
        regex=r'^document_types/(?P<document_type_id>\d+)/metadata_types/(?P<metadata_type_id>\d+)/$',
        name='documenttypemetadatatype-detail',
        view=APIDocumentTypeMetadataTypeView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/metadata/$',
        name='documentmetadata-list',
        view=APIDocumentMetadataListView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/metadata/(?P<metadata_id>\d+)/$',
        name='documentmetadata-detail',
        view=APIDocumentMetadataView.as_view()
    )
]



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('django_gpg', '0005_remove_key_key_id'),
    ]

    operations = [
        migrations.AlterField(
            model_name='key',
            name='key_data',
            field=models.TextField(
                help_text='ASCII armored version of the key.',
                verbose_name='Key data'
            ),
        ),
    ]



from mayan.apps.testing.tests.base import GenericViewTestCase

from ..events import event_workflow_template_edited
from ..permissions import (
    permission_workflow_template_edit, permission_workflow_template_view
)

from .mixins.workflow_template_mixins import WorkflowTemplateTestMixin
from .mixins.workflow_template_transition_mixins import (
    WorkflowTransitionFieldTestMixin, WorkflowTransitionFieldViewTestMixin,
    WorkflowTemplateTransitionViewTestMixin
)


class WorkflowTransitionFieldViewTestCase(
    WorkflowTemplateTestMixin, WorkflowTransitionFieldTestMixin,
    WorkflowTransitionFieldViewTestMixin, WorkflowTemplateTransitionViewTestMixin,
    GenericViewTestCase
):
    def setUp(self):
        super().setUp()
        self._create_test_workflow_template()
        self._create_test_workflow_template_state()
        self._create_test_workflow_template_state()
        self._create_test_workflow_template_transition()

    def test_workflow_template_transition_field_create_view_no_permission(self):
        workflow_template_transition_field_count = self._test_workflow_template_transition.fields.count()

        self._clear_events()

        response = self._request_workflow_template_transition_field_create_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            self._test_workflow_template_transition.fields.count(),
            workflow_template_transition_field_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_transition_field_create_view_with_access(self):
        workflow_template_transition_field_count = self._test_workflow_template_transition.fields.count()
        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_workflow_template_transition_field_create_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self._test_workflow_template_transition.fields.count(),
            workflow_template_transition_field_count + 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(
            events[0].action_object,
            self._test_workflow_template_transition_field
        )
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_workflow_template)
        self.assertEqual(events[0].verb, event_workflow_template_edited.id)

    def test_workflow_template_transition_field_delete_view_no_permission(self):
        self._create_test_workflow_template_transition_field()
        workflow_template_transition_field_count = self._test_workflow_template_transition.fields.count()

        self._clear_events()

        response = self._request_workflow_template_transition_field_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            self._test_workflow_template_transition.fields.count(),
            workflow_template_transition_field_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_transition_field_delete_view_with_access(self):
        self._create_test_workflow_template_transition_field()
        workflow_template_transition_field_count = self._test_workflow_template_transition.fields.count()

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_workflow_template_transition_field_delete_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self._test_workflow_template_transition.fields.count(),
            workflow_template_transition_field_count - 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_workflow_template)
        self.assertEqual(events[0].verb, event_workflow_template_edited.id)

    def test_workflow_template_transition_field_edit_view_no_permission(self):
        self._create_test_workflow_template_transition_field()
        workflow_template_transition_field_label = self._test_workflow_template_transition_field.label

        self._clear_events()

        response = self._request_workflow_template_transition_field_edit_view()
        self.assertEqual(response.status_code, 404)

        self._test_workflow_template_transition_field.refresh_from_db()
        self.assertEqual(
            workflow_template_transition_field_label,
            self._test_workflow_template_transition_field.label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_transition_field_edit_view_with_access(self):
        self._create_test_workflow_template_transition_field()
        workflow_template_transition_field_label = self._test_workflow_template_transition_field.label

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_workflow_template_transition_field_edit_view()
        self.assertEqual(response.status_code, 302)

        self._test_workflow_template_transition_field.refresh_from_db()
        self.assertNotEqual(
            workflow_template_transition_field_label,
            self._test_workflow_template_transition_field.label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(
            events[0].action_object,
            self._test_workflow_template_transition_field
        )
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_workflow_template)
        self.assertEqual(events[0].verb, event_workflow_template_edited.id)

    def test_workflow_template_transition_field_list_view_no_permission(self):
        self._create_test_workflow_template_transition_field()

        self._clear_events()

        response = self._request_test_workflow_template_transition_field_list_view()
        self.assertNotContains(
            response=response,
            text=self._test_workflow_template_transition_field.label,
            status_code=404
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_transition_field_list_view_with_access(self):
        self._create_test_workflow_template_transition_field()
        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_view
        )

        self._clear_events()

        response = self._request_test_workflow_template_transition_field_list_view()
        self.assertContains(
            response=response,
            text=self._test_workflow_template_transition_field.label,
            status_code=200
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



import logging

from django.http import HttpResponse

from mayan.apps.rest_api import generics
from mayan.apps.rest_api.api_view_mixins import ExternalContentTypeObjectAPIViewMixin

from .api_view_mixins import APIImageViewMixin
from .classes import AppImageErrorImage
from .models import Asset
from .permissions import (
    permission_asset_create, permission_asset_delete, permission_asset_edit,
    permission_asset_view
)
from .serializers import AssetSerializer

logger = logging.getLogger(name=__name__)


class APIAssetListView(generics.ListCreateAPIView):
    """
    get: Returns a list of all the assets.
    post: Create a new asset.
    """
    mayan_object_permissions = {'GET': (permission_asset_view,)}
    mayan_view_permissions = {'POST': (permission_asset_create,)}
    ordering_fields = ('id', 'internal_name', 'label')
    queryset = Asset.objects.all()
    serializer_class = AssetSerializer

    def get_instance_extra_data(self):
        return {
            '_event_actor': self.request.user
        }


class APIAssetDetailView(generics.RetrieveUpdateDestroyAPIView):
    """
    delete: Delete the selected asset.
    get: Return the details of the selected asset.
    patch: Edit the properties of the selected asset.
    put: Edit the properties of the selected asset.
    """
    lookup_url_kwarg = 'asset_id'
    mayan_object_permissions = {
        'DELETE': (permission_asset_delete,),
        'GET': (permission_asset_view,),
        'PATCH': (permission_asset_edit,),
        'PUT': (permission_asset_edit,)
    }
    queryset = Asset.objects.all()
    serializer_class = AssetSerializer

    def get_instance_extra_data(self):
        return {
            '_event_actor': self.request.user
        }


class APIAssetImageView(APIImageViewMixin, generics.RetrieveAPIView):
    """
    get: Returns an image representation of the selected asset.
    """
    lookup_url_kwarg = 'asset_id'
    mayan_object_permissions = {
        'GET': (permission_asset_view,),
    }
    queryset = Asset.objects.all()


class APIAppImageErrorImageView(generics.RetrieveAPIView):
    """
    get: Returns an image representation of the selected app image error.
    """
    lookup_url_kwarg = 'app_image_error_name'

    def get_object(self):
        return AppImageErrorImage.get(
            name=self.kwargs[self.lookup_url_kwarg]
        )

    def get_serializer(self, *args, **kwargs):
        return None

    def get_serializer_class(self):
        return None

    def retrieve(self, request, *args, **kwargs):
        with self.get_object().open() as file_object:
            response = HttpResponse(content=file_object.read(), content_type='image')
            return response


class APIContentObjectImageView(
    APIImageViewMixin, ExternalContentTypeObjectAPIViewMixin,
    generics.RetrieveAPIView
):
    """
    get: Returns an image representation of the selected content object.
    """
    def get_content_type(self):
        return self.content_type

    def set_object(self):
        self.obj = self.external_object



from mayan.apps.documents.tests.mixins.document_mixins import DocumentTestMixin
from mayan.apps.testing.tests.base import BaseTestCase

from ..forms import DocumentMetadataForm

from .mixins import MetadataTypeTestMixin


class DocumentMetadataFormTestCase(
    DocumentTestMixin, MetadataTypeTestMixin, BaseTestCase
):
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()

        self._create_test_document_stub()
        self._create_test_metadata_type(
            add_test_document_type=True, required=True
        )

    def test_document_metadata_form_empty_required(self):
        form = DocumentMetadataForm(
            data={'update': True},
            initial={
                'document_type': self.test_document_type,
                'metadata_type': self.test_metadata_type
            }
        )

        # Trigger clean method.
        errors = form.errors

        self.assertEqual(
            errors['value'], [
                '"{}" is required for this document type.'.format(
                    self.test_metadata_type.label
                )
            ]
        )
        self.assertEqual(
            errors['metadata_type_id'], ['This field is required.']
        )

    def test_document_metadata_form_required_with_value_no_checkbox(self):
        form = DocumentMetadataForm(
            data={
                'update': False,
                'value': 'test value'
            },
            initial={
                'document_type': self.test_document_type,
                'metadata_type': self.test_metadata_type
            }
        )

        # Trigger clean method.
        errors = form.errors

        self.assertEqual(
            errors['value'], [
                '"{}" is required for this document type.'.format(
                    self.test_metadata_type.label
                )
            ]
        )
        self.assertEqual(
            errors['metadata_type_id'], ['This field is required.']
        )



from mayan.apps.acls.models import AccessControlList
from mayan.apps.permissions.models import StoredPermission


class RelatedObjectPermissionViewMixin:
    def get_source_queryset(self):
        queryset = super().get_source_queryset()
        permission_values = queryset.values('permission')
        stored_permission_queryset = StoredPermission.objects.filter(
            pk__in=permission_values
        )

        result = queryset.none()
        for stored_permission in stored_permission_queryset:
            result = result | AccessControlList.objects.restrict_queryset(
                permission=stored_permission.volatile_permission,
                queryset=queryset, user=self.request.user
            )

        result = result | queryset.filter(permission__isnull=True)
        return result.distinct()



from django.apps import apps


def handler_initialize_new_user_otp_data(sender, instance, **kwargs):
    UserOTPData = apps.get_model(
        app_label='authentication_otp', model_name='UserOTPData'
    )

    if kwargs['created']:
        UserOTPData.objects.create(user=instance)



from PIL import Image

from mayan.apps.documents.tests.base import GenericDocumentTestCase

from ..layers import layer_decorations
from ..transformations import TransformationAssetPastePercent

from .mixins import AssetTestMixin


class DecorationTestCase(AssetTestMixin, GenericDocumentTestCase):
    auto_create_test_transformation_class = False

    def _get_test_document_version_page_image(self):
        cache_filename = self.test_document_version_page.generate_image()
        cache_file = self.test_document_version_page.cache_partition.get_file(
            filename=cache_filename
        )
        with cache_file.open() as file_object:
            image = Image.open(fp=file_object)
            image.load()
            return image

    def test_asset_paste_percent_transformation(self):
        self._create_test_asset()

        test_asset_pixel_color = self.test_asset.get_image().getpixel(
            xy=(0, 0)
        )[0:3]

        test_document_version_page_image = self._get_test_document_version_page_image()
        self.assertNotEqual(
            test_document_version_page_image.getpixel(xy=(0, 0)),
            test_asset_pixel_color
        )

        self.assertNotEqual(
            test_document_version_page_image.getpixel(
                xy=(
                    test_document_version_page_image.size[0] / 2,
                    test_document_version_page_image.size[1] / 2
                )
            ), test_asset_pixel_color
        )

        layer_decorations.add_transformation_to(
            obj=self.test_document_version_page,
            transformation_class=TransformationAssetPastePercent,
            arguments={
                'asset_name': self.test_asset.internal_name,
                'left': 50,
                'top': 50
            }
        )

        transformations = layer_decorations.get_transformations_for(
            as_classes=True, obj=self.test_document_version_page
        )

        test_asset_images = transformations[0].get_asset_images()
        position_x = test_asset_images['image_asset'].size[0] / 2
        position_y = test_asset_images['image_asset'].size[1] / 2

        test_document_version_page_image = self._get_test_document_version_page_image()

        self.assertNotEqual(
            test_document_version_page_image.getpixel(
                xy=(position_x, position_y)
            ), test_asset_pixel_color
        )

        self.assertEqual(
            test_document_version_page_image.getpixel(
                xy=(
                    test_document_version_page_image.size[0] / 2,
                    test_document_version_page_image.size[1] / 2
                )
            ), test_asset_pixel_color
        )



import mayan.apps.common.models
import django.core.files.storage
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('common', '0009_auto_20180402_0339'),
    ]

    operations = [
        migrations.AlterField(
            model_name='shareduploadedfile',
            name='file',
            field=models.FileField(
                storage=django.core.files.storage.FileSystemStorage(
                    location=b'mayan/media/shared_files'
                ), upload_to=mayan.apps.common.models.upload_to,
                verbose_name='File'
            ),
        ),
    ]



from django.apps import apps
from django.contrib.auth import get_user_model
from django.db import models
from django.utils.translation import ugettext_lazy as _

from mayan.apps.acls.classes import ModelPermission
from mayan.apps.common.apps import MayanAppConfig
from mayan.apps.common.menus import (
    menu_list_facet, menu_object, menu_secondary, menu_tools, menu_topbar
)
from mayan.apps.navigation.classes import SourceColumn
from mayan.apps.views.html_widgets import ObjectLinkWidget, TwoStateWidget

from .classes import EventTypeNamespace
from .html_widgets import widget_event_actor_link, widget_event_type_link
from .links import (
    link_event_types_subscriptions_list, link_events_for_object_clear,
    link_events_for_object_export, link_events_list, link_events_list_clear,
    link_events_list_export, link_notification_mark_read,
    link_notification_mark_read_all, link_user_notifications_list
)


class EventsApp(MayanAppConfig):
    app_namespace = 'events'
    app_url = 'events'
    has_rest_api = True
    has_tests = True
    name = 'mayan.apps.events'
    verbose_name = _('Events')

    def ready(self):
        super().ready()

        Action = apps.get_model(app_label='actstream', model_name='Action')
        Notification = self.get_model(model_name='Notification')
        StoredEventType = self.get_model(model_name='StoredEventType')

        User = get_user_model()

        EventTypeNamespace.load_modules()

        # Typecast the related field because actstream uses CharFields for
        # the object_id the action_object, actor, and target fields.
        ModelPermission.register_inheritance(
            fk_field_cast=models.CharField, model=Action,
            related='action_object'
        )
        ModelPermission.register_inheritance(
            fk_field_cast=models.CharField, model=Action, related='actor'
        )
        ModelPermission.register_inheritance(
            fk_field_cast=models.CharField, model=Action, related='target'
        )

        # Add labels to Action model, they are not marked translatable in the
        # upstream package.
        SourceColumn(
            attribute='timestamp', is_identifier=True,
            is_sortable=True, label=_('Date and time'), name='timestamp',
            source=Action
        )
        SourceColumn(
            func=widget_event_actor_link, label=_('Actor'),
            include_label=True, source=Action
        )
        SourceColumn(
            func=widget_event_type_link, label=_('Event'),
            include_label=True, name='event_type', source=Action
        )
        SourceColumn(
            attribute='target', label=_('Target'), include_label=True,
            name='target', source=Action, widget=ObjectLinkWidget
        )
        SourceColumn(
            attribute='action_object', label=_('Action object'),
            include_label=True, source=Action, widget=ObjectLinkWidget
        )

        SourceColumn(
            source=StoredEventType, label=_('Namespace'), attribute='namespace'
        )
        SourceColumn(
            source=StoredEventType, label=_('Label'), attribute='label'
        )

        SourceColumn(
            attribute='action__timestamp', is_identifier=True,
            is_sortable=True, label=_('Date and time'), source=Notification
        )
        SourceColumn(
            func=widget_event_actor_link, label=_('Actor'),
            include_label=True, kwargs={'attribute': 'action'},
            source=Notification
        )
        SourceColumn(
            func=widget_event_type_link, label=_('Event'),
            include_label=True, kwargs={'attribute': 'action'},
            source=Notification
        )
        SourceColumn(
            attribute='action.target', label=_('Target'), include_label=True,
            source=Notification, widget=ObjectLinkWidget
        )
        SourceColumn(
            attribute='action.action_object', label=_('Action object'),
            include_label=True, source=Notification, widget=ObjectLinkWidget
        )
        SourceColumn(
            attribute='read', include_label=True, is_sortable=True,
            label=_('Seen'), source=Notification, widget=TwoStateWidget
        )

        # Clear

        menu_secondary.bind_links(
            links=(link_events_list_clear,),
            sources=(
                'events:events_list',
                'events:events_list_clear',
            )
        )
        menu_secondary.bind_links(
            links=(link_events_for_object_clear,),
            sources=(
                'events:events_for_object',
                'events:events_for_object_clear'
            )
        )

        # Export

        menu_secondary.bind_links(
            links=(link_events_list_export,),
            sources=(
                'events:events_list',
                'events:events_list_export',
            )
        )
        menu_secondary.bind_links(
            links=(link_events_for_object_export,),
            sources=(
                'events:events_for_object',
                'events:events_for_object_export'
            )
        )

        # Notification

        menu_object.bind_links(
            links=(link_notification_mark_read,), sources=(Notification,)
        )

        menu_secondary.bind_links(
            links=(link_notification_mark_read_all,),
            sources=(
                'events:notification_mark_read',
                'events:notification_mark_read_all',
                'events:user_notifications_list'
            )
        )

        # Subscription

        menu_list_facet.bind_links(
            links=(
                link_event_types_subscriptions_list,
            ), sources=(User,)
        )

        # Other

        menu_topbar.bind_links(
            links=(link_user_notifications_list,), position=30
        )
        menu_tools.bind_links(links=(link_events_list,))



from django.utils.translation import ugettext_lazy as _

from mayan.apps.navigation.classes import Link

from .icons import icon_dashboard_detail, icon_dashboard_list


link_dashboard_list = Link(
    icon=icon_dashboard_list, text=_('Dashboards'), view='dashboards:dashboard_list'
)

link_dashboard_detail = Link(
    args='object.name', icon=icon_dashboard_detail,
    text=_('Details'), view='dashboards:dashboard_detail'
)



from mayan.apps.documents.tests.base import GenericDocumentTestCase

from .literals import TEST_DOCUMENT_CONTENT
from .mixins import DocumentFileContentTestMixin


class DocumentFileContentMethodTestCase(
    DocumentFileContentTestMixin, GenericDocumentTestCase
):
    auto_upload_test_document = False

    def test_document_file_parsed_content(self):
        self._upload_test_document()

        self._create_test_document_file_parsed_content()
        value = ' '.join(self.test_document_file.content())

        self.assertEqual(value, TEST_DOCUMENT_CONTENT)

    def test_document_parsed_content(self):
        self._upload_test_document()
        self._create_test_document_file_parsed_content()
        value = ' '.join(self.test_document.content())

        self.assertEqual(value, TEST_DOCUMENT_CONTENT)

    def test_document_stub_parsed_content(self):
        self._create_test_document_stub()

        value = ' '.join(self.test_document.content())

        self.assertFalse(value)



from django.utils.translation import ugettext_lazy as _

from mayan.apps.documents.search import (
    document_version_search, document_version_page_search, document_search
)

# Document

document_search.add_model_field(
    field='versions__version_pages__ocr_content__content',
    label=_('Document version OCR')
)

# Document version

document_version_search.add_model_field(
    field='version_pages__ocr_content__content', label=_('OCR')
)

# Document version page

document_version_page_search.add_model_field(
    field='ocr_content__content', label=_('Document version OCR')
)



from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('documents', '0072_auto_20201128_0919'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='DuplicatedDocument',
            new_name='DuplicatedDocumentOld',
        ),
    ]



from mayan.apps.testing.tests.base import GenericViewTestCase

from ..permissions import (
    permission_group_create, permission_group_view, permission_user_create,
    permission_user_view
)

from .mixins import GroupLinkTestMixin, GroupTestMixin, UserLinkTestMixin


class GroupLinkTestCase(
    GroupLinkTestMixin, GroupTestMixin, GenericViewTestCase
):
    def test_group_setup_link_no_permission(self):
        resolved_link = self._resolve_group_setup_link()
        self.assertEqual(resolved_link, None)

    def test_group_setup_link_with_create_permission(self):
        self.grant_permission(permission=permission_group_create)
        resolved_link = self._resolve_group_setup_link()
        self.assertNotEqual(resolved_link, None)

    def test_group_setup_link_with_view_access(self):
        self._create_test_group()

        self.grant_access(
            obj=self.test_group, permission=permission_group_view
        )
        resolved_link = self._resolve_group_setup_link()
        self.assertNotEqual(resolved_link, None)


class UserLinkTestCase(UserLinkTestMixin, GenericViewTestCase):
    def test_user_setup_link_no_permission(self):
        resolved_link = self._resolve_user_setup_link()
        self.assertEqual(resolved_link, None)

    def test_user_setup_link_with_create_permission(self):
        self.grant_permission(permission=permission_user_create)
        resolved_link = self._resolve_user_setup_link()
        self.assertNotEqual(resolved_link, None)

    def test_user_setup_link_with_view_access(self):
        self._create_test_user()

        self.grant_access(
            obj=self.test_user, permission=permission_user_view
        )
        resolved_link = self._resolve_user_setup_link()
        self.assertNotEqual(resolved_link, None)



from .. import *  # NOQA

ALLOWED_HOSTS = ['*']

DEBUG = True

CELERY_BROKER_URL = 'memory://'
CELERY_TASK_ALWAYS_EAGER = True
CELERY_TASK_EAGER_PROPAGATES = CELERY_TASK_ALWAYS_EAGER  # NOQA: F405
EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'

if 'rosetta' not in INSTALLED_APPS:   # NOQA: F405
    try:
        import rosetta  # NOQA: F401
    except ImportError:
        pass
    else:
        INSTALLED_APPS += (  # NOQA: F405
            'rosetta',
        )

if 'django_extensions' not in INSTALLED_APPS:
    try:
        import django_extensions  # NOQA: F401
    except ImportError:
        pass
    else:
        INSTALLED_APPS += (
            'django_extensions',
        )

# Allow using WhiteNoise in development.
INSTALLED_APPS.insert(0, 'whitenoise.runserver_nostatic')

LOGGING_LEVEL = 'DEBUG'
ROOT_URLCONF = 'mayan.urls.development'

SEARCH_BACKEND = 'mayan.apps.dynamic_search.backends.django.DjangoSearchBackend'

TEMPLATES[0]['OPTIONS']['loaders'] = (  # NOQA: F405
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
)

WSGI_AUTO_RELOAD = True



from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('documents', '0067_auto_20201024_1120'),
    ]

    operations = [
        migrations.DeleteModel(
            name='DeletedDocument',
        ),
        migrations.RenameField(
            model_name='document',
            old_name='deleted_date_time',
            new_name='trashed_date_time',
        ),
    ]



from django import forms

from mayan.apps.views.forms import DetailForm

from .models import Comment


class DocumentCommentDetailForm(DetailForm):

    class Meta:
        fields = ('text',)
        extra_fields = (
            {'field': 'submit_date', 'widget': forms.widgets.DateTimeInput},
            {'field': 'user'},
        )
        model = Comment



from django.db.models import Q

from ..models import Key

from .literals import TEST_KEY_PRIVATE_DATA, TEST_KEY_PUBLIC_FILE_PATH


class KeyAPIViewTestMixin:
    def _request_test_key_create_api_view(self):
        pk_list = list(Key.objects.values_list('pk', flat=True))

        response = self.post(
            viewname='rest_api:key-list', data={
                'key_data': TEST_KEY_PRIVATE_DATA
            }
        )

        try:
            self.test_key_private = Key.objects.get(
                ~Q(pk__in=pk_list)
            )
        except Key.DoesNotExist:
            self.test_key_private = None

        return response

    def _request_test_key_delete_api_view(self):
        return self.delete(
            viewname='rest_api:key-detail', kwargs={
                'key_id': self.test_key_private.pk
            }
        )

    def _request_test_key_detail_api_view(self):
        return self.get(
            viewname='rest_api:key-detail', kwargs={
                'key_id': self.test_key_private.pk
            }
        )


class KeyTestMixin:
    def _create_test_key_private(self):
        self.test_key_private = Key.objects.create(
            key_data=TEST_KEY_PRIVATE_DATA
        )

    def _create_test_key_public(self):
        with open(file=TEST_KEY_PUBLIC_FILE_PATH, mode='rb') as file_object:
            self.test_key_public = Key.objects.create(
                key_data=file_object.read()
            )


class KeyViewTestMixin:
    def _request_test_key_delete_view(self):
        return self.post(
            viewname='django_gpg:key_delete', kwargs={
                'key_id': self.test_key_private.pk
            }
        )

    def _request_test_key_download_view(self):
        return self.get(
            viewname='django_gpg:key_download', kwargs={
                'key_id': self.test_key_private.pk
            }
        )

    def _request_test_key_upload_view(self):
        pk_list = list(Key.objects.values_list('pk', flat=True))

        response = self.post(
            viewname='django_gpg:key_upload', data={
                'key_data': TEST_KEY_PRIVATE_DATA
            }
        )

        try:
            self.test_key_private = Key.objects.get(
                ~Q(pk__in=pk_list)
            )
        except Key.DoesNotExist:
            self.test_key_private = None

        return response



from mayan.apps.testing.tests.base import BaseTestCase

from .mixins import MessageTestMixin


class MessageModelTestCase(MessageTestMixin, BaseTestCase):
    def test_method_get_absolute_url(self):
        self._create_test_message()

        self.assertTrue(self.test_message.get_absolute_url())



from mayan.apps.common.tests.mixins import ObjectCopyTestMixin
from mayan.apps.documents.tests.mixins.document_mixins import DocumentTestMixin
from mayan.apps.testing.tests.base import BaseTestCase

from .mixins import WebLinkTestMixin


class WebLinkCopyTestCase(
    DocumentTestMixin, WebLinkTestMixin, ObjectCopyTestMixin, BaseTestCase
):
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()
        self._create_test_web_link()
        self.test_web_link.document_types.add(self.test_document_type)
        self.test_object = self.test_web_link



from django.utils.translation import ugettext_lazy as _

from mayan.apps.documents.models.document_file_models import DocumentFile

from ..permissions import (
    permission_send_document_file_attachment,
    permission_send_document_file_link
)

from .base import ObjectAttachmentMailView, ObjectLinkMailView


class MailDocumentFileAttachmentView(ObjectAttachmentMailView):
    object_permission = permission_send_document_file_attachment
    pk_url_kwarg = 'document_file_id'
    source_queryset = DocumentFile.valid.all()
    success_message = _('%(count)d document file queued for email delivery')
    success_message_plural = _(
        '%(count)d document files queued for email delivery'
    )
    title = 'Email document file'
    title_plural = 'Email document files'
    title_document = 'Email document file: %s'


class MailDocumentFileLinkView(ObjectLinkMailView):
    object_permission = permission_send_document_file_link
    pk_url_kwarg = 'document_file_id'
    source_queryset = DocumentFile.valid.all()
    success_message = _(
        '%(count)d document file link queued for email delivery'
    )
    success_message_plural = _(
        '%(count)d document file links queued for email delivery'
    )
    title = 'Email document file link'
    title_plural = 'Email document file links'
    title_document = 'Email link for document file: %s'



from django.core import management

from ...utils import PyPIClient


class Command(management.BaseCommand):
    help = 'Check if the current version is the latest.'

    def handle(self, *args, **options):
        self.stdout.write(
            msg='{}\n'.format(PyPIClient().check_version_verbose())
        )



from django.contrib import messages
from django.utils.translation import ugettext_lazy as _

from mayan.apps.views.generics import ConfirmView, SingleObjectListView
from mayan.apps.views.mixins import ExternalContentTypeObjectViewMixin

from .icons import icon_object_errors
from .models import GlobalErrorLogPartitionEntry
from .permissions import permission_error_log_view


class GlobalErrorLogEntryList(SingleObjectListView):
    model = GlobalErrorLogPartitionEntry
    object_permission = permission_error_log_view

    def get_extra_context(self):
        return {
            'hide_object': True,
            'no_results_icon': icon_object_errors,
            'no_results_text': _(
                'This view displays the error log of different objects. '
                'An empty list is a good thing.'
            ),
            'no_results_title': _(
                'There are no error log entries'
            ),
            'title': _('Global error log')
        }


class ObjectErrorLogEntryListClearView(
    ExternalContentTypeObjectViewMixin, ConfirmView
):
    external_object_permission = permission_error_log_view

    def get_extra_context(self):
        return {
            'object': self.external_object,
            'title': _(
                'Clear error log entries for: %s' % self.external_object
            ),
        }

    def view_action(self):
        self.external_object.error_log.all().delete()
        messages.success(
            message=_('Object error log cleared successfully'),
            request=self.request
        )


class ObjectErrorLogEntryListView(
    ExternalContentTypeObjectViewMixin, SingleObjectListView
):
    external_object_permission = permission_error_log_view

    def get_extra_context(self):
        return {
            'hide_object': True,
            'no_results_icon': icon_object_errors,
            'no_results_text': _(
                'This view displays the error log of an object. '
                'An empty list is a good thing.'
            ),
            'no_results_title': _(
                'There are no error log entries'
            ),
            'object': self.external_object,
            'title': _('Error log entries for: %s' % self.external_object),
        }

    def get_source_queryset(self):
        return self.external_object.error_log.all()



from pathlib import Path
import uuid

from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.translation import ugettext_lazy as _

from mayan.apps.databases.model_mixins import ExtraDataModelMixin
from mayan.apps.events.classes import EventManagerMethodAfter, EventManagerSave
from mayan.apps.events.decorators import method_event
from mayan.apps.permissions.models import StoredPermission

from .classes import DefinedStorageLazy
from .events import (
    event_download_file_created, event_download_file_deleted,
    event_download_file_downloaded
)
from .literals import (
    STORAGE_NAME_DOWNLOAD_FILE, STORAGE_NAME_SHARED_UPLOADED_FILE
)
from .managers import DownloadFileManager, SharedUploadedFileManager
from .model_mixins import DatabaseFileModelMixin


def download_file_upload_to(instance, filename):
    return 'download-file-{}'.format(uuid.uuid4().hex)


def upload_to(instance, filename):
    return 'shared-file-{}'.format(uuid.uuid4().hex)


class DownloadFile(DatabaseFileModelMixin, ExtraDataModelMixin, models.Model):
    """
    Keep a database link to a stored file. Used for generates files meant
    to be downloaded at a later time.
    """
    file = models.FileField(
        storage=DefinedStorageLazy(
            name=STORAGE_NAME_DOWNLOAD_FILE
        ), upload_to=download_file_upload_to, verbose_name=_('File')
    )
    content_type = models.ForeignKey(
        blank=True, null=True, on_delete=models.CASCADE, to=ContentType
    )
    object_id = models.PositiveIntegerField(blank=True, null=True)
    content_object = GenericForeignKey(
        ct_field='content_type', fk_field='object_id'
    )
    label = models.CharField(
        db_index=True, max_length=192, verbose_name=_('Label')
    )
    permission = models.ForeignKey(
        blank=True, null=True, on_delete=models.CASCADE,
        to=StoredPermission, verbose_name=_('Permission')
    )

    objects = DownloadFileManager()

    class Meta:
        ordering = ('-datetime',)
        verbose_name = _('Download file')
        verbose_name_plural = _('Download files')

    def __str__(self):
        if self.content_object:
            return str(self.content_object)
        else:
            return self.filename or self.label

    @method_event(
        event_manager_class=EventManagerMethodAfter,
        event=event_download_file_deleted,
        target='content_object'
    )
    def delete(self, *args, **kwargs):
        return super().delete(*args, **kwargs)

    def get_absolute_url(self):
        if self.content_object:
            return self.content_object.get_absolute_url()

    def get_absolute_api_url(self):
        if self.content_object:
            return self.content_object.get_absolute_api_url()

    @method_event(
        event_manager_class=EventManagerMethodAfter,
        event=event_download_file_downloaded,
        target='self'
    )
    def get_download_file_object(self):
        return self.open(mode='rb')

    @method_event(
        event_manager_class=EventManagerSave,
        created={
            'action_object': 'content_object',
            'event': event_download_file_created,
            'target': 'self'
        }
    )
    def save(self, *args, **kwargs):
        super().save(*args, **kwargs)


class SharedUploadedFile(DatabaseFileModelMixin, models.Model):
    """
    Keep a database link to a stored file. Used to share files between code
    that runs out of process.
    """
    file = models.FileField(
        storage=DefinedStorageLazy(
            name=STORAGE_NAME_SHARED_UPLOADED_FILE
        ), upload_to=upload_to, verbose_name=_('File')
    )
    filename = models.CharField(
        blank=True, max_length=255, verbose_name=_('Filename')
    )

    objects = SharedUploadedFileManager()

    class Meta:
        verbose_name = _('Shared uploaded file')
        verbose_name_plural = _('Shared uploaded files')

    def __str__(self):
        return self.filename

    def save(self, *args, **kwargs):
        self.filename = self.filename or Path(path=self.file.name).name
        super().save(*args, **kwargs)
        self.file.close()



from mayan.apps.testing.tests.base import BaseTestCase
from mayan.celery import app as celery_app

from ..classes import CeleryQueue

from .mixins import TaskManagerTestMixin, TaskManagerViewTestMixin


class CeleryQueueTestCase(
    TaskManagerTestMixin, TaskManagerViewTestMixin, BaseTestCase
):
    def test_queue_creation(self):
        CeleryQueue.update_celery()

        test_celery_queue_count = len(celery_app.conf.task_queues)

        self._create_test_queue()
        CeleryQueue.update_celery()

        self.assertTrue(
            len(celery_app.conf.task_queues), test_celery_queue_count + 1
        )

    def test_queue_removal(self):
        self._create_test_queue(
            label='test-queue-unique-1', name='test-queue-unique-1'
        )
        CeleryQueue.update_celery()
        test_celery_queue_count = len(celery_app.conf.task_queues)

        self._test_queue.remove()
        CeleryQueue.update_celery()

        self.assertTrue(
            len(celery_app.conf.task_queues), test_celery_queue_count - 1
        )



from unittest import skip

from django.test import override_settings

from mayan.apps.testing.tests.base import BaseTestCase

from .mixins import (
    LockBackendManagementCommandTestCaseMixin, LockBackendTestMixin
)


class FileLockBackendManagementCommandTestCase(
    LockBackendTestMixin, LockBackendManagementCommandTestCaseMixin,
    BaseTestCase
):
    test_locking_backend_string = 'mayan.apps.lock_manager.backends.file_lock.FileLock'


class ModelLockBackendManagementCommandTestCase(
    LockBackendTestMixin, LockBackendManagementCommandTestCaseMixin,
    BaseTestCase
):
    test_locking_backend_string = 'mayan.apps.lock_manager.backends.model_lock.ModelLock'


@skip('Skip until a Mock Redis server class is added.')
@override_settings(
    LOCK_MANAGER_BACKEND_ARGUMENTS={'redis_url': 'redis://127.0.0.1:6379/0'}
)
class RedisLockBackendManagementCommandTestCase(
    LockBackendTestMixin, LockBackendManagementCommandTestCaseMixin,
    BaseTestCase
):
    test_locking_backend_string = 'mayan.apps.lock_manager.backends.redis_lock.RedisLock'



from mayan.apps.acls.models import AccessControlList
from mayan.apps.documents.models.document_models import Document
from mayan.apps.documents.permissions import permission_document_view
from mayan.apps.rest_api import generics

from .models import DocumentCheckout
from .permissions import (
    permission_document_check_in, permission_document_check_in_override,
    permission_document_check_out_detail_view
)
from .serializers import (
    DocumentCheckoutSerializer, NewDocumentCheckoutSerializer
)


class APICheckedoutDocumentListView(generics.ListCreateAPIView):
    """
    get: Returns a list of all the documents that are currently checked out.
    post: Checkout a document.
    """
    def get_serializer(self, *args, **kwargs):
        if not self.request:
            return None

        return super().get_serializer(*args, **kwargs)

    def get_serializer_class(self):
        if self.request.method == 'POST':
            return NewDocumentCheckoutSerializer
        else:
            return DocumentCheckoutSerializer

    def get_queryset(self):
        valid_document_queryset = Document.valid.all()

        filtered_documents = AccessControlList.objects.restrict_queryset(
            permission=permission_document_view,
            queryset=DocumentCheckout.objects.checked_out_documents(),
            user=self.request.user,
        )
        filtered_documents = AccessControlList.objects.restrict_queryset(
            permission=permission_document_check_out_detail_view,
            queryset=filtered_documents,
            user=self.request.user,
        )

        return DocumentCheckout.objects.filter(
            document__pk__in=filtered_documents.filter(
                pk__in=valid_document_queryset.values('pk')
            ).values('pk')
        )


class APICheckedoutDocumentView(generics.RetrieveDestroyAPIView):
    """
    get: Retrieve the details of the selected checked out document entry.
    delete: Checkin a document.
    """
    lookup_url_kwarg = 'checkout_id'
    serializer_class = DocumentCheckoutSerializer

    def get_instance_extra_data(self):
        return {
            '_event_actor': self.request.user,
            '_event_keep_attributes': ('_event_actor',)
        }

    def get_queryset(self):
        valid_document_queryset = Document.valid.all()

        if self.request.method == 'GET':
            filtered_documents = AccessControlList.objects.restrict_queryset(
                permission=permission_document_view,
                queryset=DocumentCheckout.objects.checked_out_documents(),
                user=self.request.user
            )
            filtered_documents = AccessControlList.objects.restrict_queryset(
                permission=permission_document_check_out_detail_view,
                queryset=filtered_documents, user=self.request.user
            )

            return DocumentCheckout.objects.filter(
                document__pk__in=filtered_documents.filter(
                    pk__in=valid_document_queryset.values('pk')
                ).values('pk')
            )
        elif self.request.method == 'DELETE':
            check_in_queryset = AccessControlList.objects.restrict_queryset(
                permission=permission_document_check_in,
                queryset=DocumentCheckout.objects.filter(user_id=self.request.user.pk),
                user=self.request.user
            )
            check_in_override_queryset = AccessControlList.objects.restrict_queryset(
                permission=permission_document_check_in_override,
                queryset=DocumentCheckout.objects.exclude(
                    user_id=self.request.user.pk
                ),
                user=self.request.user
            )

            return (check_in_queryset | check_in_override_queryset).filter(
                document_id__in=valid_document_queryset.values('pk')
            )



import pyotp


class AuthenticationOTPTestMixin:
    def _enable_test_otp(self):
        self._test_secret = pyotp.random_base32()
        self._test_totp = pyotp.TOTP(self._test_secret)

        self._test_token = self._test_totp.now()
        self._test_case_superuser.otp_data.enable(
            secret=self._test_secret, token=self._test_token
        )



from django.contrib import admin

from .models import (
    DocumentTypeOCRSettings, DocumentVersionOCRError,
    DocumentVersionPageOCRContent
)


@admin.register(DocumentTypeOCRSettings)
class DocumentTypeOCRSettingsAdmin(admin.ModelAdmin):
    list_display = ('document_type', 'auto_ocr')


@admin.register(DocumentVersionOCRError)
class DocumentVersionOCRErrorAdmin(admin.ModelAdmin):
    list_display = ('document_version', 'datetime_submitted')
    readonly_fields = ('document_version', 'datetime_submitted', 'result')


@admin.register(DocumentVersionPageOCRContent)
class DocumentVersionPageOCRContentAdmin(admin.ModelAdmin):
    list_display = ('document_version_page',)



import shutil

from mayan.apps.documents.tests.literals import TEST_SMALL_DOCUMENT_PATH
from mayan.apps.storage.utils import fs_cleanup, mkdtemp

from ...source_backends.literals import (
    DEFAULT_PERIOD_INTERVAL, SOURCE_UNCOMPRESS_CHOICE_NEVER
)
from ...source_backends.watch_folder_backends import SourceBackendWatchFolder

from .base_mixins import SourceTestMixin


class WatchFolderSourceTestMixin(SourceTestMixin):
    _create_source_method = '_create_test_watch_folder'

    def setUp(self):
        self._temporary_folders = []
        super().setUp()
        self.test_staging_folder_files = []

    def tearDown(self):
        for temporary_folders in self._temporary_folders:
            fs_cleanup(filename=temporary_folders)

        super().tearDown()

    def _create_test_watch_folder(self, extra_data=None):
        temporary_folder = mkdtemp()
        self._temporary_folders.append(temporary_folder)
        backend_data = {
            'document_type_id': self.test_document_type.pk,
            'folder_path': temporary_folder,
            'include_subdirectories': False,
            'interval': DEFAULT_PERIOD_INTERVAL,
            'uncompress': SOURCE_UNCOMPRESS_CHOICE_NEVER
        }

        if extra_data:
            backend_data.update(extra_data)

        self._create_test_source(
            backend_path=SourceBackendWatchFolder.get_class_path(),
            backend_data=backend_data
        )

    def _copy_test_watch_folder_document(self):
        shutil.copy(
            src=TEST_SMALL_DOCUMENT_PATH,
            dst=self.test_source.get_backend_data()['folder_path']
        )



from mayan.apps.testing.tests.base import GenericViewTestCase

from ..events import event_workflow_template_edited
from ..models import WorkflowTransition
from ..permissions import (
    permission_workflow_template_edit, permission_workflow_template_view
)

from .literals import TEST_WORKFLOW_TEMPLATE_TRANSITION_LABEL
from .mixins.workflow_template_mixins import (
    WorkflowTemplateTestMixin, WorkflowTemplateViewTestMixin
)
from .mixins.workflow_template_transition_mixins import (
    WorkflowTemplateTransitionViewTestMixin
)


class WorkflowTransitionViewTestCase(
    WorkflowTemplateTestMixin, WorkflowTemplateViewTestMixin,
    WorkflowTemplateTransitionViewTestMixin, GenericViewTestCase
):
    def setUp(self):
        super().setUp()
        self._create_test_workflow_template()
        self._create_test_workflow_template_state()
        self._create_test_workflow_template_state()

    def test_workflow_template_transition_create_view_no_permission(self):
        self._clear_events()

        response = self._request_test_workflow_template_transition_create_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(WorkflowTransition.objects.count(), 0)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_transition_create_view_with_access(self):
        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_test_workflow_template_transition_create_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(WorkflowTransition.objects.count(), 1)
        self.assertEqual(
            WorkflowTransition.objects.all()[0].label,
            TEST_WORKFLOW_TEMPLATE_TRANSITION_LABEL
        )
        self.assertEqual(
            WorkflowTransition.objects.all()[0].origin_state,
            self._test_workflow_template_states[0]
        )
        self.assertEqual(
            WorkflowTransition.objects.all()[0].destination_state,
            self._test_workflow_template_states[1]
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(
            events[0].action_object, self._test_workflow_template_transition
        )
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_workflow_template)
        self.assertEqual(events[0].verb, event_workflow_template_edited.id)

    def test_workflow_template_transition_delete_view_no_permission(self):
        self._create_test_workflow_template_transition()

        self._clear_events()

        response = self._request_test_workflow_template_transition_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertTrue(
            self._test_workflow_template_transition in WorkflowTransition.objects.all()
        )
        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_transition_delete_view_with_access(self):
        self._create_test_workflow_template_transition()

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_test_workflow_template_transition_delete_view()
        self.assertEqual(response.status_code, 302)

        self.assertFalse(
            self._test_workflow_template_transition in WorkflowTransition.objects.all()
        )
        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_workflow_template)
        self.assertEqual(events[0].verb, event_workflow_template_edited.id)

    def test_workflow_template_transition_edit_view_no_permission(self):
        self._create_test_workflow_template_transition()

        test_workflow_template_transition_label = self._test_workflow_template_transition.label

        self._clear_events()

        response = self._request_test_workflow_template_transition_edit_view()
        self.assertEqual(response.status_code, 404)

        self._test_workflow_template_transition.refresh_from_db()
        self.assertEqual(
            self._test_workflow_template_transition.label,
            test_workflow_template_transition_label
        )
        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_transition_edit_view_with_access(self):
        self._create_test_workflow_template_transition()

        test_workflow_template_transition_label = self._test_workflow_template_transition.label

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_test_workflow_template_transition_edit_view()
        self.assertEqual(response.status_code, 302)

        self._test_workflow_template_transition.refresh_from_db()
        self.assertNotEqual(
            self._test_workflow_template_transition.label,
            test_workflow_template_transition_label
        )
        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(
            events[0].action_object, self._test_workflow_template_transition
        )
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_workflow_template)
        self.assertEqual(events[0].verb, event_workflow_template_edited.id)

    def test_workflow_template_transition_list_view_no_permission(self):
        self._create_test_workflow_template_transition()

        self._clear_events()

        response = self._request_test_workflow_template_transition_list_view()
        self.assertNotContains(
            response=response,
            text=self._test_workflow_template_transition.label,
            status_code=404
        )
        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_transition_list_view_with_access(self):
        self._create_test_workflow_template_transition()

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_view
        )

        self._clear_events()

        response = self._request_test_workflow_template_transition_list_view()
        self.assertContains(
            response=response,
            text=self._test_workflow_template_transition.label,
            status_code=200
        )
        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



from django.utils.translation import ugettext_lazy as _

from mayan.apps.storage.classes import DefinedStorage

from .literals import STORAGE_NAME_ASSETS, STORAGE_NAME_ASSETS_CACHE
from .settings import (
    setting_asset_cache_storage_backend,
    setting_asset_cache_storage_backend_arguments,
    setting_asset_storage_backend, setting_asset_storage_backend_arguments
)

storage_assets = DefinedStorage(
    dotted_path=setting_asset_storage_backend.value,
    error_message=_(
        'Unable to initialize the converter asset '
        'storage. Check the settings {} and {} for formatting '
        'errors.'.format(
            setting_asset_storage_backend.global_name,
            setting_asset_storage_backend_arguments.global_name
        )
    ), label=_('Assets'), name=STORAGE_NAME_ASSETS,
    kwargs=setting_asset_storage_backend_arguments.value
)

storage_assets_cache = DefinedStorage(
    dotted_path=setting_asset_cache_storage_backend.value,
    error_message=_(
        'Unable to initialize the converter asset cache'
        'storage. Check the settings {} and {} for formatting '
        'errors.'.format(
            setting_asset_cache_storage_backend.global_name,
            setting_asset_cache_storage_backend_arguments.global_name
        )
    ), label=_('Assets cache'), name=STORAGE_NAME_ASSETS_CACHE,
    kwargs=setting_asset_cache_storage_backend_arguments.value
)



from django.db.models import Q

from ...models import WorkflowState, WorkflowStateAction

from ..literals import (
    DOCUMENT_WORKFLOW_LAUNCH_ACTION_CLASS_PATH,
    TEST_WORKFLOW_TEMPLATE_STATE_ACTION_GENERIC_DOTTED_PATH,
    TEST_WORKFLOW_TEMPLATE_STATE_ACTION_LABEL,
    TEST_WORKFLOW_TEMPLATE_STATE_ACTION_LABEL_EDITED,
    TEST_WORKFLOW_TEMPLATE_STATE_ACTION_DOTTED_PATH,
    TEST_WORKFLOW_TEMPLATE_STATE_ACTION_WHEN,
    TEST_WORKFLOW_TEMPLATE_STATE_COMPLETION,
    TEST_WORKFLOW_TEMPLATE_STATE_LABEL,
    TEST_WORKFLOW_TEMPLATE_STATE_LABEL_EDITED
)
from ..workflow_actions import TestWorkflowAction


class WorkflowTemplateStateActionAPIViewTestMixin:
    def _request_test_workflow_template_state_action_create_api_view(self):
        pk_list = list(WorkflowStateAction.objects.values('pk'))

        total_test_workflow_template_state_actions = len(
            self._test_workflow_template_state_actions
        )
        label = '{}_{}'.format(
            TEST_WORKFLOW_TEMPLATE_STATE_ACTION_LABEL,
            total_test_workflow_template_state_actions
        )

        data = {
            'label': label,
            'action_path': self._test_workflow_template_state_action_path
        }

        response = self.post(
            viewname='rest_api:workflow-template-state-action-list',
            kwargs={
                'workflow_template_id': self._test_workflow_template.pk,
                'workflow_template_state_id': self._test_workflow_template_state.pk
            }, data=data
        )

        try:
            self._test_workflow_template_state_action = WorkflowStateAction.objects.get(
                ~Q(pk__in=pk_list)
            )
        except WorkflowStateAction.DoesNotExist:
            self._test_workflow_template_state_action = None

        return response

    def _request_test_workflow_template_state_action_delete_api_view(self):
        return self.delete(
            viewname='rest_api:workflow-template-state-action-detail',
            kwargs={
                'workflow_template_id': self._test_workflow_template.pk,
                'workflow_template_state_id': self._test_workflow_template_state.pk,
                'workflow_template_state_action_id': self._test_workflow_template_state_action.pk
            }
        )

    def _request_test_workflow_template_state_action_detail_api_view(self):
        return self.get(
            viewname='rest_api:workflow-template-state-action-detail',
            kwargs={
                'workflow_template_id': self._test_workflow_template.pk,
                'workflow_template_state_id': self._test_workflow_template_state.pk,
                'workflow_template_state_action_id': self._test_workflow_template_state_action.pk
            }
        )

    def _request_test_workflow_template_state_action_list_api_view(self):
        return self.get(
            viewname='rest_api:workflow-template-state-action-list', kwargs={
                'workflow_template_id': self._test_workflow_template.pk,
                'workflow_template_state_id': self._test_workflow_template_state.pk
            }
        )

    def _request_test_workflow_template_state_action_edit_patch_api_view(self):
        return self.patch(
            viewname='rest_api:workflow-template-state-action-detail',
            kwargs={
                'workflow_template_id': self._test_workflow_template.pk,
                'workflow_template_state_id': self._test_workflow_template_state.pk,
                'workflow_template_state_action_id': self._test_workflow_template_state_action.pk
            }, data={
                'label': TEST_WORKFLOW_TEMPLATE_STATE_ACTION_LABEL_EDITED
            }
        )

    def _request_test_workflow_template_state_action_edit_put_api_view(self):
        return self.put(
            viewname='rest_api:workflow-template-state-action-detail',
            kwargs={
                'workflow_template_id': self._test_workflow_template.pk,
                'workflow_template_state_id': self._test_workflow_template_state.pk,
                'workflow_template_state_action_id': self._test_workflow_template_state_action.pk
            }, data={
                'label': TEST_WORKFLOW_TEMPLATE_STATE_ACTION_LABEL_EDITED,
                'action_path': self._test_workflow_template_state_action_path
            }
        )


class WorkflowTemplateStateActionLaunchViewTestMixin:
    def _request_document_workflow_template_launch_action_create_view(self):
        return self.post(
            viewname='document_states:workflow_template_state_action_create',
            kwargs={
                'workflow_template_state_id': self._test_workflow_template_state.pk,
                'class_path': DOCUMENT_WORKFLOW_LAUNCH_ACTION_CLASS_PATH
            }, data={
                'enabled': True,
                'label': TEST_WORKFLOW_TEMPLATE_STATE_ACTION_LABEL,
                'when': TEST_WORKFLOW_TEMPLATE_STATE_ACTION_WHEN,
                'workflows': self._test_workflow_templates[0].pk
            }
        )


class WorkflowTemplateStateActionTestMixin:
    TestWorkflowAction = TestWorkflowAction
    _test_workflow_template_state_action_path = TEST_WORKFLOW_TEMPLATE_STATE_ACTION_GENERIC_DOTTED_PATH

    def setUp(self):
        super().setUp()
        self._test_workflow_template_state_actions = []

    def _create_test_workflow_template_state_action(
        self, extra_data=None, workflow_state_index=0
    ):
        total_test_workflow_template_state_actions = len(
            self._test_workflow_template_state_actions
        )
        label = '{}_{}'.format(
            TEST_WORKFLOW_TEMPLATE_STATE_ACTION_LABEL,
            total_test_workflow_template_state_actions
        )

        data = {
            'label': label,
            'action_path': self._test_workflow_template_state_action_path
        }

        if extra_data:
            data.update(extra_data)

        self._test_workflow_template_state_action = self._test_workflow_template_states[
            workflow_state_index
        ].actions.create(
            **data
        )

        self._test_workflow_template_state_actions.append(
            self._test_workflow_template_state_action
        )


class WorkflowTemplateStateActionViewTestMixin:
    def _request_test_workflow_template_state_action_create_get_view(self, class_path):
        return self.get(
            viewname='document_states:workflow_template_state_action_create',
            kwargs={
                'workflow_template_state_id': self._test_workflow_template_state.pk,
                'class_path': class_path
            }
        )

    def _request_test_workflow_template_state_action_create_post_view(
        self, class_path, extra_data=None
    ):
        data = {
            'label': TEST_WORKFLOW_TEMPLATE_STATE_ACTION_LABEL,
            'when': TEST_WORKFLOW_TEMPLATE_STATE_ACTION_WHEN
        }
        if extra_data:
            data.update(extra_data)

        pk_list = list(
            WorkflowStateAction.objects.values_list('pk', flat=True)
        )

        response = self.post(
            viewname='document_states:workflow_template_state_action_create',
            kwargs={
                'workflow_template_state_id': self._test_workflow_template_state.pk,
                'class_path': class_path
            }, data=data
        )

        try:
            self._test_workflow_template_state_action = WorkflowStateAction.objects.get(
                ~Q(pk__in=pk_list)
            )
        except WorkflowStateAction.DoesNotExist:
            self._test_workflow_template_state_action = None

        return response

    def _request_test_worflow_template_state_action_delete_view(self):
        return self.post(
            viewname='document_states:workflow_template_state_action_delete',
            kwargs={
                'workflow_template_state_action_id': self._test_workflow_template_state_action.pk
            }
        )

    def _request_test_worflow_template_state_action_edit_view(self):
        return self.post(
            viewname='document_states:workflow_template_state_action_edit',
            kwargs={
                'workflow_template_state_action_id': self._test_workflow_template_state_action.pk
            }, data={
                'label': TEST_WORKFLOW_TEMPLATE_STATE_ACTION_LABEL_EDITED,
                'when': self._test_workflow_template_state_action.when
            }
        )

    def _request_test_worflow_template_state_action_list_view(self):
        return self.get(
            viewname='document_states:workflow_template_state_action_list',
            kwargs={
                'workflow_template_state_id': self._test_workflow_template_state.pk
            }
        )

    def _request_test_workflow_template_state_action_selection_view(self):
        return self.post(
            viewname='document_states:workflow_template_state_action_selection',
            kwargs={
                'workflow_template_state_id': self._test_workflow_template_state.pk
            }, data={
                'klass': TEST_WORKFLOW_TEMPLATE_STATE_ACTION_DOTTED_PATH
            }
        )


class WorkflowTemplateStateAPIViewTestMixin:
    def _request_test_workflow_template_state_create_api_view(self):
        pk_list = list(WorkflowState.objects.values('pk'))

        response = self.post(
            viewname='rest_api:workflow-template-state-list',
            kwargs={
                'workflow_template_id': self._test_workflow_template.pk
            }, data={
                'completion': TEST_WORKFLOW_TEMPLATE_STATE_COMPLETION,
                'label': TEST_WORKFLOW_TEMPLATE_STATE_LABEL
            }
        )

        try:
            self._test_workflow_template_state = WorkflowState.objects.get(
                ~Q(pk__in=pk_list)
            )
        except WorkflowState.DoesNotExist:
            self._test_workflow_template_state = None

        return response

    def _request_test_workflow_template_state_delete_api_view(self):
        return self.delete(
            viewname='rest_api:workflow-template-state-detail',
            kwargs={
                'workflow_template_id': self._test_workflow_template.pk,
                'workflow_template_state_id': self._test_workflow_template_state.pk
            }
        )

    def _request_test_workflow_template_state_detail_api_view(self):
        return self.get(
            viewname='rest_api:workflow-template-state-detail',
            kwargs={
                'workflow_template_id': self._test_workflow_template.pk,
                'workflow_template_state_id': self._test_workflow_template_state.pk
            }
        )

    def _request_test_workflow_template_state_list_api_view(self):
        return self.get(
            viewname='rest_api:workflow-template-state-list', kwargs={
                'workflow_template_id': self._test_workflow_template.pk
            }
        )

    def _request_test_workflow_template_state_edit_patch_api_view(self):
        return self.patch(
            viewname='rest_api:workflow-template-state-detail',
            kwargs={
                'workflow_template_id': self._test_workflow_template.pk,
                'workflow_template_state_id': self._test_workflow_template_state.pk
            }, data={
                'label': TEST_WORKFLOW_TEMPLATE_STATE_LABEL_EDITED
            }
        )

    def _request_test_workflow_template_state_edit_put_api_view(self):
        return self.put(
            viewname='rest_api:workflow-template-state-detail',
            kwargs={
                'workflow_template_id': self._test_workflow_template.pk,
                'workflow_template_state_id': self._test_workflow_template_state.pk
            }, data={
                'label': TEST_WORKFLOW_TEMPLATE_STATE_LABEL_EDITED
            }
        )


class WorkflowStateViewTestMixin:
    def _request_test_workflow_template_state_create_view(self, extra_data=None):
        pk_list = list(WorkflowState.objects.values_list('pk', flat=True))

        data = {
            'label': TEST_WORKFLOW_TEMPLATE_STATE_LABEL,
            'completion': TEST_WORKFLOW_TEMPLATE_STATE_COMPLETION,
        }
        if extra_data:
            data.update(extra_data)

        response = self.post(
            viewname='document_states:workflow_template_state_create',
            kwargs={
                'workflow_template_id': self._test_workflow_template.pk
            }, data=data
        )

        try:
            self._test_workflow_template_state = WorkflowState.objects.get(
                ~Q(pk__in=pk_list)
            )
        except WorkflowState.DoesNotExist:
            self._test_workflow_template_state = None

        return response

    def _request_test_workflow_template_state_delete_view(self):
        return self.post(
            viewname='document_states:workflow_template_state_delete',
            kwargs={
                'workflow_template_state_id': self._test_workflow_template_states[0].pk
            }
        )

    def _request_test_workflow_template_state_edit_view(self):
        return self.post(
            viewname='document_states:workflow_template_state_edit',
            kwargs={
                'workflow_template_state_id': self._test_workflow_template_states[0].pk
            }, data={
                'label': TEST_WORKFLOW_TEMPLATE_STATE_LABEL_EDITED
            }
        )

    def _request_test_workflow_template_state_list_view(self):
        return self.get(
            viewname='document_states:workflow_template_state_list',
            kwargs={'workflow_template_id': self._test_workflow_template.pk}
        )



from django.utils.translation import ugettext_lazy as _

from mayan.apps.views.generics import SingleObjectListView, SimpleView

from .classes import Dashboard


class DashboardListView(SingleObjectListView):
    extra_context = {
        'hide_object': True,
        'title': _('Dashboards'),
    }

    def get_source_queryset(self):
        return Dashboard.get_all()


class DashboardDetailView(SimpleView):
    template_name = 'appearance/generic_template.html'

    def get_extra_context(self):
        dashboard = Dashboard.get(name=self.kwargs['dashboard_name'])

        return {
            'content': dashboard.render(request=self.request),
            'object': dashboard,
            'title': _('Dashboard detail')
        }



from django.db import models, migrations


class Migration(migrations.Migration):
    dependencies = [
        ('contenttypes', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='PermissionHolder',
            fields=[
                (
                    'id', models.AutoField(
                        verbose_name='ID', serialize=False, auto_created=True,
                        primary_key=True
                    )
                ),
                ('holder_id', models.PositiveIntegerField()),
                (
                    'holder_type', models.ForeignKey(
                        on_delete=models.CASCADE,
                        related_name='permission_holder',
                        to='contenttypes.ContentType'
                    )
                ),
            ],
            options={
                'verbose_name': 'Permission holder',
                'verbose_name_plural': 'Permission holders',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Role',
            fields=[
                (
                    'id', models.AutoField(
                        verbose_name='ID', serialize=False, auto_created=True,
                        primary_key=True
                    )
                ),
                ('name', models.CharField(unique=True, max_length=64)),
                (
                    'label', models.CharField(
                        unique=True, max_length=64, verbose_name='Label'
                    )
                ),
            ],
            options={
                'ordering': ('label',),
                'verbose_name': 'Role',
                'verbose_name_plural': 'Roles',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='RoleMember',
            fields=[
                (
                    'id', models.AutoField(
                        verbose_name='ID', serialize=False, auto_created=True,
                        primary_key=True
                    )
                ),
                ('member_id', models.PositiveIntegerField()),
                (
                    'member_type', models.ForeignKey(
                        on_delete=models.CASCADE, related_name='role_member',
                        to='contenttypes.ContentType'
                    )
                ),
                (
                    'role', models.ForeignKey(
                        on_delete=models.CASCADE, to='permissions.Role',
                        verbose_name='Role'
                    )
                ),
            ],
            options={
                'verbose_name': 'Role member',
                'verbose_name_plural': 'Role members',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='StoredPermission',
            fields=[
                (
                    'id', models.AutoField(
                        verbose_name='ID', serialize=False, auto_created=True,
                        primary_key=True
                    )
                ),
                (
                    'namespace', models.CharField(
                        max_length=64, verbose_name='Namespace'
                    )
                ),
                ('name', models.CharField(max_length=64, verbose_name='Name')),
            ],
            options={
                'ordering': ('namespace',),
                'verbose_name': 'Permission',
                'verbose_name_plural': 'Permissions',
            },
            bases=(models.Model,),
        ),
        migrations.AlterUniqueTogether(
            name='storedpermission',
            unique_together=set([('namespace', 'name')]),
        ),
        migrations.AddField(
            model_name='permissionholder',
            name='permission',
            field=models.ForeignKey(
                on_delete=models.CASCADE, to='permissions.StoredPermission',
                verbose_name='Permission'
            ),
            preserve_default=True,
        ),
    ]
